@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from ibm_watson_machine_learning.utils.log_util import get_logger
 from ibm_watson_machine_learning.utils import version
@@ -36,6 +41,7 @@ from ibm_watson_machine_learning.wml_client_error import WMLClientError
 from ibm_watson_machine_learning.instance_new_plan import ServiceInstanceNewPlan
 
 from ibm_watson_machine_learning.migration_v4ga_cloud import Migrationv4GACloud
+from ibm_watson_machine_learning.remote_training_system import RemoteTrainingSystem
 
 import copy
 import os
@@ -68,6 +74,8 @@ class APIClient:
         self.CAMS_URL = None
         self.CREATED_IN_V1_PLAN = False
         self.version_param = '2020-08-01'
+        self.ICP_PLATFORM_SPACES = False # This will be applicable for 3.5 and later and specific to convergence functionalities
+        self.ICP_35 = False # Use it for any 3.5 specific functionalities
         # self.version_param = '2020-06-12'
 
         self.PLATFORM_URLS_MAP = {
@@ -91,7 +99,9 @@ class APIClient:
 
         self.project_id = project_id
         self.wml_token = None
-        self.__wml_local_supported_version_list = ['1.0', '1.1', '2.0', '2.0.1', '2.0.2', '2.5.0', '3.0.0', '3.0.1']
+        self.__wml_local_supported_version_list = ['1.0', '1.1',
+                                                   '2.0', '2.0.1', '2.0.2', '2.5.0',
+                                                   '3.0.0', '3.0.1', '3.5']
         self.__wsd_supported_version_list = ['1.0', '1.1', '2.0']
         self.__predefined_instance_type_list = ['icp', 'openshift', 'wml_local', 'wsd_local']
         os.environ['WSD_PLATFORM'] = 'False'
@@ -124,7 +134,7 @@ class APIClient:
                         '2.0.1' == wml_credentials[u'version'].lower() or '2.5.0' == wml_credentials[
                     u'version'].lower() or
                         '3.0.0' == wml_credentials[u'version'].lower() or '3.0.1' == wml_credentials[
-                            u'version'].lower() or
+                            u'version'].lower() or '3.5' == wml_credentials[u'version'].lower() or
                         '1.1' == wml_credentials[u'version'].lower() or '2.0' == wml_credentials[u'version'].lower()):
                     self.CAMS = True
                     os.environ["DEPLOYMENT_PRIVATE"] = "icp4d"
@@ -140,6 +150,11 @@ class APIClient:
                             '2.0' == wml_credentials[u'version'].lower():
                         self.ICP_30 = True
 
+                    # For Cloud convergence related functionalities brought into CP4D 3.5
+                    if '3.5' == wml_credentials[u'version'].lower():
+                        self.ICP_PLATFORM_SPACES = True
+                        self.ICP_35 = True
+
                 else:
                     if 'version' in wml_credentials.keys() and \
                             wml_credentials[u'version'].lower() not in self.__wml_local_supported_version_list:
@@ -258,17 +273,25 @@ class APIClient:
         self.wml_token = None
 
        # if not self.ICP and not self.ICP_30 and not self.WSD:
-        if not self.WSD and not self.CLOUD_PLATFORM_SPACES:
+        if not self.WSD and not self.CLOUD_PLATFORM_SPACES and not self.ICP_PLATFORM_SPACES:
             self.service_instance = ServiceInstance(self)
             self.service_instance.details = self.service_instance.get_details()
 
-        if self.CLOUD_PLATFORM_SPACES:
+        if self.CLOUD_PLATFORM_SPACES or self.ICP_PLATFORM_SPACES:
+            # For cloud, service_instance.details will be set during space creation( if instance is associated ) or
+            # while patching a space with an instance
             self.service_instance = ServiceInstanceNewPlan(self)
+
+            if self.ICP_PLATFORM_SPACES:
+                self.service_instance.details = self.service_instance.get_details()
+
             self.set = Set(self)
 
             self.spaces = PlatformSpaces(self)
 
-            self.shiny = Shiny(self)
+            if self.ICP_PLATFORM_SPACES:
+                self.shiny = Shiny(self)
+
             self.script = Script(self)
             self.model_definitions = ModelDefinition(self)
 
@@ -276,6 +299,7 @@ class APIClient:
             self.software_specifications = SwSpec(self)
 
             self.hardware_specifications = HwSpec(self)
+
             self.connections = Connections(self)
             self.repository = Repository(self)
             self._functions = Functions(self)
@@ -285,10 +309,13 @@ class APIClient:
 
             self.deployments = Deployments(self)
 
-            self.v4ga_cloud_migration = Migrationv4GACloud(self)
+            if self.CLOUD_PLATFORM_SPACES:
+                self.v4ga_cloud_migration = Migrationv4GACloud(self)
+
+            self.remote_training_systems = RemoteTrainingSystem(self)
 
         ##Initialize Assets and Model_Definitions only for CAMS
-        if self.CAMS or self.WSD:
+        if (self.CAMS or self.WSD) and not self.ICP_PLATFORM_SPACES:
             self.set = Set(self)
             self.data_assets = Assets(self)
             self.model_definitions = ModelDefinition(self)
@@ -310,7 +337,7 @@ class APIClient:
         self.experiments = Experiments(self)
         self._functions = Functions(self)
 
-        if not self.WSD and not self.CLOUD_PLATFORM_SPACES:
+        if not self.WSD and not self.CLOUD_PLATFORM_SPACES and not self.ICP_PLATFORM_SPACES:
             self.runtimes = Runtimes(self)
             self.deployments = Deployments(self)
             self.training = Training(self)
@@ -334,7 +361,7 @@ class APIClient:
     def _params(self, skip_space_project_chk=False):
         params = {}
         if self.CAMS or self.CLOUD_PLATFORM_SPACES:
-            if self.CLOUD_PLATFORM_SPACES:
+            if self.CLOUD_PLATFORM_SPACES or self.ICP_PLATFORM_SPACES:
                 params.update({'version': self.version_param})
 
             if self.default_space_id is not None:
@@ -377,7 +404,7 @@ class APIClient:
                 'X-WML-User-Client': 'PythonClient'
             }
             # Cloud Convergence
-            if self._is_IAM() or (self.service_instance._is_iam() is None and not self.CLOUD_PLATFORM_SPACES):
+            if self._is_IAM() or (self.service_instance._is_iam() is None and not self.CLOUD_PLATFORM_SPACES and not self.ICP_PLATFORM_SPACES):
                 headers['ML-Instance-ID'] = self.wml_credentials['instance_id']
 
             headers.update({'x-wml-internal-switch-to-new-v4': "true"})
