@@ -1,20 +1,37 @@
-from abc import ABC, abstractmethod
-from functools import wraps
-from typing import TYPE_CHECKING, Union, Dict, Tuple
-from copy import copy
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-from ibm_watson_machine_learning.experiment import AutoAI
-from ibm_watson_machine_learning.utils.deployment.errors import (
-    WrongDeploymnetType, ModelTypeNotSupported, NotAutoAIExperiment, ModelStoringFailed, DeploymentNotSupported)
-from ibm_watson_machine_learning.utils.autoai.utils import (
-    prepare_auto_ai_model_to_publish, remove_file, prepare_auto_ai_model_to_publish_notebook,
-    download_experiment_details_from_file, download_wml_pipeline_details_from_file)
+from abc import ABC, abstractmethod
+from copy import copy
+from functools import wraps
+from typing import TYPE_CHECKING, Union, Dict
+from warnings import warn
+from time import sleep
 
 from pandas import DataFrame
 
+from ..experiment import AutoAI
+from ..utils.autoai.utils import (prepare_auto_ai_model_to_publish, prepare_auto_ai_model_to_publish_normal_scenario,
+                                  remove_file, prepare_auto_ai_model_to_publish_notebook,
+                                  prepare_auto_ai_model_to_publish_notebook_normal_scenario,
+                                  download_wml_pipeline_details_from_file, get_sw_spec_and_type_based_on_sklearn)
+from ..utils.deployment.errors import (WrongDeploymnetType, ModelTypeNotSupported, NotAutoAIExperiment,
+                                       DeploymentNotSupported, MissingSpace, ModelStoringFailed)
+from ..wml_client_error import ApiRequestFailure
+from ..workspace import WorkSpace
+
 if TYPE_CHECKING:
-    from ibm_watson_machine_learning.helpers.connections import DataConnection
-    from ibm_boto3 import resource
     from sklearn.pipeline import Pipeline
     from numpy import ndarray
 
@@ -28,15 +45,85 @@ class BaseDeployment(ABC):
     Base abstract class for Deployment.
     """
 
-    def __init__(self, **kwargs):
-        self._source_workspace = None
-        self._target_workspace = None
+    def __init__(self,
+                 deployment_type: str,
+                 source_wml_credentials: Union[dict, 'WorkSpace'] = None,
+                 source_project_id: str = None,
+                 source_space_id: str = None,
+                 target_wml_credentials: Union[dict, 'WorkSpace'] = None,
+                 target_project_id: str = None,
+                 target_space_id: str = None,
+                 wml_credentials: Union[dict, 'WorkSpace'] = None,
+                 project_id: str = None,
+                 space_id: str = None):
+
+        if space_id is None and source_space_id is None and target_space_id is None:
+            raise MissingSpace(reason="Any of the [space_id, source_space_id, target_space_id] is not specified.")
+
+        # note: backward compatibility
+        if wml_credentials is not None:
+            source_wml_credentials = wml_credentials
+            warn("\"wml_credentials\" parameter is depreciated, please use \"source_wml_credentials\"")
+            print("\"wml_credentials\" parameter is depreciated, please use \"source_wml_credentials\"")
+
+        if project_id is not None:
+            source_project_id = project_id
+            warn("\"project_id\" parameter is depreciated, please use \"source_project_id\"")
+            print("\"project_id\" parameter is depreciated, please use \"source_project_id\"")
+
+        if space_id is not None:
+            source_space_id = space_id
+            warn("\"space_id\" parameter is depreciated, please use \"source_space_id\"")
+            print("\"space_id\" parameter is depreciated, please use \"source_space_id\"")
+        # --- end note
+
+        # note: as workspace is not clear enough to understand, there is a possibility to use pure
+        # wml credentials with project and space IDs, but in addition we
+        # leave a possibility to use a previous WorkSpace implementation, it could be passed as a first argument
+        if isinstance(source_wml_credentials, WorkSpace):
+            self._source_workspace = source_wml_credentials
+
+        elif isinstance(source_wml_credentials, dict):
+            self._source_workspace = WorkSpace(wml_credentials=source_wml_credentials.copy(),
+                                               project_id=source_project_id,
+                                               space_id=source_space_id)
+        else:
+            self._source_workspace = None
+
+        if target_wml_credentials is None:
+            if isinstance(source_wml_credentials, WorkSpace):
+                self._target_workspace = source_wml_credentials
+
+            elif isinstance(source_wml_credentials, dict):
+                self._target_workspace = WorkSpace(wml_credentials=source_wml_credentials.copy(),
+                                                   project_id=source_project_id,
+                                                   space_id=source_space_id)
+            else:
+                self._target_workspace = None
+
+        else:
+            if isinstance(target_wml_credentials, WorkSpace):
+                self._target_workspace = target_wml_credentials
+
+            elif isinstance(target_wml_credentials, dict):
+                self._target_workspace = WorkSpace(wml_credentials=target_wml_credentials.copy(),
+                                                   project_id=target_project_id,
+                                                   space_id=target_space_id)
+
+                # note: only if user provides target WML information
+                if self._source_workspace is None:
+                    self._source_workspace = copy(self._target_workspace)
+
+            else:
+                self._target_workspace = None
+        # --- end note
+
         self.name = None
         self.id = None
-        if kwargs['deployment_type'] == 'online':
+        if deployment_type == 'online':
             self.scoring_url = None
 
-        if kwargs['deployment_type'] == 'batch':
+        if deployment_type == 'batch':
             self._obm = False
 
     def __repr__(self):
@@ -52,8 +139,8 @@ class BaseDeployment(ABC):
 
         Parameters
         ----------
-        model: Union[Any, str], required
-            Model object to deploy or local path to the model.
+        model: str, required
+            AutoAI model name.
 
         deployment_name: str, required
             Name of the deployment
@@ -93,57 +180,58 @@ class BaseDeployment(ABC):
                 self._obm = False
 
             print("Preparing an AutoAI Deployment...")
+            # TODO: remove part with model object depployment
+            if not isinstance(kwargs['model'], str):
+                warning_msg = ("Depreciation Warning: Passing an object will no longer be supported. "
+                               "Please specify the AutoAI model name to deploy.")
+                print(warning_msg)
+                warn(warning_msg)
+            # note: check if model is of lale type, if yes, convert it back to scikit
+            try:
+                model = kwargs['model'].export_to_sklearn_pipeline()
 
-            optimizer = AutoAI(self._source_workspace).runs.get_optimizer(run_id=kwargs['experiment_run_id'])
+            except AttributeError:
+                model = kwargs['model']
+            # --- end note
 
-            # note: check if user passed a pipeline name instead of object,
-            # applicable only for pipelines from experiment
-            if isinstance(kwargs['model'], str):
-                model = optimizer.get_pipeline(pipeline_name=kwargs['model'], astype=AutoAI.PipelineTypes.SKLEARN)
-            else:
-                # note: check if model is of lale type, if yes, convert it back to scikit
-                try:
-                    model = kwargs['model'].export_to_sklearn_pipeline()
+            # note: check if model is of lale type, if yes, convert it back to scikit
+            if not isinstance(kwargs['model'], str):
+                model_props = {
+                    self._target_workspace.wml_client.repository.ModelMetaNames.NAME: f"{kwargs['deployment_name']} Model",
+                    self._target_workspace.wml_client.repository.ModelMetaNames.TYPE: "wml-hybrid_0.1",
+                    self._target_workspace.wml_client.repository.ModelMetaNames.SOFTWARE_SPEC_UID:
+                        self._target_workspace.wml_client.software_specifications.get_uid_by_name("hybrid_0.1")
+                }
 
-                except AttributeError:
-                    model = kwargs['model']
+                # fill additional needed OBM metadata
+                if self._obm:
+                    model_props[self._target_workspace.wml_client.repository.ModelMetaNames.TRAINING_DATA_REFERENCES] = (
+                        run_params['entity']['training_data_references']
+                    )
                 # --- end note
-            # --- end note
 
-            # note: download training data to pass it with deployment
-            if self._obm:
-                training_data = optimizer.get_preprocessed_data_connection().read()
+                schema, artifact_name = prepare_auto_ai_model_to_publish(
+                    pipeline_model=model,
+                    run_params=run_params,
+                    run_id=kwargs['experiment_run_id'],
+                    wml_client=self._source_workspace.wml_client)
+
+                model_props[self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = (
+                    schema['schemas'] if self._obm else [schema])
 
             else:
-                training_data = optimizer.get_data_connections()[0].read()
-
-            training_target = training_data[optimizer.params['prediction_column']]
-            training_data = training_data.drop([optimizer.params['prediction_column']], axis=1)
-            # --- end note
-
-            model_props = {
-                self._target_workspace.wml_client.repository.ModelMetaNames.NAME: f"{kwargs['deployment_name']} Model",
-                self._target_workspace.wml_client.repository.ModelMetaNames.TYPE: "wml-hybrid_0.1",
-                self._target_workspace.wml_client.repository.ModelMetaNames.SOFTWARE_SPEC_UID:
-                    self._target_workspace.wml_client.software_specifications.get_uid_by_name("hybrid_0.1")
-            }
-
-            schema, artifact_name = prepare_auto_ai_model_to_publish(
-                pipeline_model=model,
-                run_params=run_params,
-                run_id=kwargs['experiment_run_id'],
-                wml_client=self._target_workspace.wml_client)
-
-            model_props[self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = [schema]
-
-            # TODO: OBM deployment will not work with cloud convergence for now
+                artifact_name, model_props = prepare_auto_ai_model_to_publish_normal_scenario(
+                    pipeline_model=model,
+                    run_params=run_params,
+                    run_id=kwargs['experiment_run_id'],
+                    wml_client=self._source_workspace.wml_client,
+                    space_id=self._target_workspace.space_id
+                )
 
             deployment_details = self._deploy(
                 pipeline_model=artifact_name,
                 deployment_name=kwargs['deployment_name'],
-                meta_props=model_props,
-                training_data=training_data,
-                training_target=training_target
+                meta_props=model_props
             )
 
             remove_file(filename=artifact_name)
@@ -160,12 +248,18 @@ class BaseDeployment(ABC):
             if kwargs.get('metadata') is not None:
                 print("Preparing an AutoAI Deployment...")
 
+                # note: WMLS
                 if self._source_workspace is not None and self._source_workspace.WMLS:
                     optimizer = AutoAI(
                         self._source_workspace.wml_credentials,
                         space_id=self._source_workspace.space_id
                     ).runs.get_optimizer(metadata=kwargs['metadata'])
 
+                # note: CP4D 3.5
+                elif self._source_workspace is not None and self._source_workspace.wml_client.ICP:
+                    optimizer = AutoAI(self._source_workspace).runs.get_optimizer(metadata=kwargs['metadata'])
+
+                # note: CLOUD
                 else:
                     optimizer = AutoAI().runs.get_optimizer(metadata=kwargs['metadata'])
 
@@ -190,31 +284,6 @@ class BaseDeployment(ABC):
                     self._obm = False
                 # --- end note
 
-                # note: check if user passed a pipeline name instead of object,
-                # applicable only for pipelines from experiment
-                if isinstance(kwargs['model'], str):
-                    model = optimizer.get_pipeline(pipeline_name=kwargs['model'], astype=AutoAI.PipelineTypes.SKLEARN)
-                else:
-                    # note: check if model is of lale type, if yes, convert it back to scikit
-                    try:
-                        model = kwargs['model'].export_to_sklearn_pipeline()
-
-                    except AttributeError:
-                        model = kwargs['model']
-                    # --- end note
-                # --- end note
-
-                # note: download training data to pass it with deployment
-                if self._obm:
-                    training_data = optimizer.get_preprocessed_data_connection().read()
-
-                else:
-                    training_data = optimizer.get_data_connections()[0].read()
-
-                training_target = training_data[optimizer.params['prediction_column']]
-                training_data = training_data.drop([optimizer.params['prediction_column']], axis=1)
-                # --- end note
-
                 # note: only when user did not pass WMLS credentials during Service initialization
                 if self._source_workspace is None:
                     self._source_workspace = copy(optimizer._workspace)
@@ -223,58 +292,106 @@ class BaseDeployment(ABC):
                     self._target_workspace = copy(optimizer._workspace)
                 # --- end note
 
-                model_props = {
-                    self._target_workspace.wml_client.repository.ModelMetaNames.NAME:
-                        f"{kwargs['deployment_name']} Model",
-                    self._target_workspace.wml_client.repository.ModelMetaNames.TYPE: "wml-hybrid_0.1",
-                    self._target_workspace.wml_client.repository.ModelMetaNames.SOFTWARE_SPEC_UID:
-                        self._target_workspace.wml_client.software_specifications.get_uid_by_name("hybrid_0.1")
-                }
+                # TODO: remove part with model object depployment
+                if not isinstance(kwargs['model'], str):
+                    warning_msg = ("Depreciation Warning: Passing an object will no longer be supported. "
+                                   "Please specify the AutoAI model name to deploy.")
+                    print(warning_msg)
+                    warn(warning_msg)
+                # note: check if model is of lale type, if yes, convert it back to scikit
+                try:
+                    model = kwargs['model'].export_to_sklearn_pipeline()
 
-                if not self._target_workspace.wml_client.ICP:
-                    schema, artifact_name = prepare_auto_ai_model_to_publish_notebook(
-                        pipeline_model=model,
-                        result_connection=optimizer._result_client[0],
-                        cos_client=optimizer._result_client[1])
-
-                    model_props[
-                        self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = [schema]
-
-                    # TODO: OBM deployment will not work with cloud convergence for now
-
-                    deployment_details = self._deploy(
-                        pipeline_model=artifact_name,
-                        deployment_name=kwargs['deployment_name'],
-                        meta_props=model_props,
-                        training_data=training_data,
-                        training_target=training_target,
-                        result_client=optimizer._result_client
-                    )
-
-                # note: WSD part
-                else:
-                    training_result_reference = kwargs['metadata'].get('training_result_reference')
-                    run_id = training_result_reference.location.path.split('/')[-3]
-                    run_params = self._source_workspace.wml_client.training.get_details(training_uid=run_id)
-
-                    schema, artifact_name = prepare_auto_ai_model_to_publish(
-                        pipeline_model=model,
-                        run_params=run_params,
-                        run_id=kwargs['experiment_run_id'],
-                        wml_client=self._target_workspace.wml_client)
-
-                    model_props[
-                        self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = [schema]
-
-                    deployment_details = self._deploy(
-                        pipeline_model=artifact_name,
-                        deployment_name=kwargs['deployment_name'],
-                        meta_props=model_props,
-                        training_data=training_data,
-                        training_target=training_target
-                    )
+                except AttributeError:
+                    model = kwargs['model']
                 # --- end note
 
+                if not isinstance(kwargs['model'], str):
+                    model_props = {
+                        self._target_workspace.wml_client.repository.ModelMetaNames.NAME: f"{kwargs['deployment_name']} Model",
+                        self._target_workspace.wml_client.repository.ModelMetaNames.TYPE: "wml-hybrid_0.1",
+                        self._target_workspace.wml_client.repository.ModelMetaNames.SOFTWARE_SPEC_UID:
+                            self._target_workspace.wml_client.software_specifications.get_uid_by_name("hybrid_0.1")
+                    }
+
+                    # note: CLOUD
+                    if not self._target_workspace.wml_client.ICP:
+                        schema, artifact_name = prepare_auto_ai_model_to_publish_notebook(
+                            pipeline_model=model,
+                            result_connection=optimizer._result_client[0],
+                            cos_client=optimizer._result_client[1],
+                            obm=True if self._obm else False
+                        )
+
+                        model_props[self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = (
+                            schema['schemas'] if self._obm else [schema])
+
+                        deployment_details = self._deploy(
+                            pipeline_model=artifact_name,
+                            deployment_name=kwargs['deployment_name'],
+                            meta_props=model_props,
+                            result_client=optimizer._result_client
+                        )
+
+                    # note: WSD part
+                    else:
+                        training_result_reference = kwargs['metadata'].get('training_result_reference')
+                        run_id = training_result_reference.location.path.split('/')[-3]
+                        run_params = self._source_workspace.wml_client.training.get_details(training_uid=run_id)
+
+                        schema, artifact_name = prepare_auto_ai_model_to_publish(
+                            pipeline_model=model,
+                            run_params=run_params,
+                            run_id=kwargs['experiment_run_id'],
+                            wml_client=self._source_workspace.wml_client)
+
+                        model_props[self._target_workspace.wml_client.repository.ModelMetaNames.INPUT_DATA_SCHEMA] = (
+                            schema['schemas'] if self._obm else [schema])
+
+                        deployment_details = self._deploy(
+                            pipeline_model=artifact_name,
+                            deployment_name=kwargs['deployment_name'],
+                            meta_props=model_props
+                        )
+                    # --- end note
+
+                else:
+                    if not self._target_workspace.wml_client.ICP:
+                        artifact_name, model_props = prepare_auto_ai_model_to_publish_notebook_normal_scenario(
+                            pipeline_model=model,
+                            result_connection=optimizer._result_client[0],
+                            cos_client=optimizer._result_client[1],
+                            space_id=self._target_workspace.space_id
+                        )
+
+                        deployment_details = self._deploy(
+                            pipeline_model=artifact_name,
+                            deployment_name=kwargs['deployment_name'],
+                            meta_props=model_props,
+                            result_client=optimizer._result_client
+                        )
+
+                    # note: WSD / CP4D 3.5 part
+                    else:
+                        training_result_reference = kwargs['metadata'].get('training_result_reference')
+                        run_id = training_result_reference.location.path.split('/')[-3]
+                        run_params = self._source_workspace.wml_client.training.get_details(training_uid=run_id)
+
+                        artifact_name, model_props = prepare_auto_ai_model_to_publish_normal_scenario(
+                            pipeline_model=model,
+                            run_params=run_params,
+                            run_id=run_id,
+                            wml_client=self._source_workspace.wml_client,
+                            space_id=self._target_workspace.space_id
+                        )
+
+                        deployment_details = self._deploy(
+                            pipeline_model=artifact_name,
+                            deployment_name=kwargs['deployment_name'],
+                            meta_props=model_props
+                        )
+                    # --- end note
+
                 remove_file(filename=artifact_name)
 
                 self.name = kwargs['deployment_name']
@@ -431,9 +548,7 @@ class BaseDeployment(ABC):
 
     def _publish_model(self,
                        pipeline_model: Union['Pipeline', str],
-                       meta_props: Dict,
-                       training_data: Union['DataFrame', 'ndarray'],
-                       training_target: Union['DataFrame', 'ndarray']) -> str:
+                       meta_props: Dict) -> str:
         """
         Publish model into WML.
 
@@ -442,12 +557,6 @@ class BaseDeployment(ABC):
         pipeline_model: Pipeline or str, required
             Model of the pipeline to publish
 
-        training_data: Union['pandas.DataFrame', 'numpy.ndarray'], required
-            Training data for the model
-
-        training_target: Union['pandas.DataFrame', 'numpy.ndarray'], required
-            Target/label data for the model
-
         meta_props: dictionary, required
             Model meta properties.
 
@@ -457,14 +566,12 @@ class BaseDeployment(ABC):
         """
         published_model_details = self._target_workspace.wml_client.repository.store_model(
             model=pipeline_model,
-            training_data=training_data,
-            training_target=training_target,
             meta_props=meta_props)
 
-        asset_uid = self._target_workspace.wml_client.repository.get_model_uid(published_model_details)
+        asset_id = self._target_workspace.wml_client.repository.get_model_uid(published_model_details)
 
-        print(f"Published model uid: {asset_uid}")
-        return asset_uid
+        print(f"Published model uid: {asset_id}")
+        return asset_id
 
     @staticmethod
     def _project_to_space_to_project(method):
