@@ -1,25 +1,37 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import io
 import time
 from typing import TYPE_CHECKING, Any, Dict, Union, List, Optional
-from warnings import warn
 
 from pandas import DataFrame, concat
 
 from .base_deployment import BaseDeployment
 from ..helpers import DataConnection
 from ..utils import StatusLogger, print_text_header_h1
+from ..utils.autoai.connection import validate_source_data_connections, validate_deployment_output_connection
 from ..utils.autoai.utils import init_cos_client, try_load_dataset
-from ..utils.autoai.watson_studio import change_client_get_headers
+from ..utils.autoai.errors import NoneDataConnection
 from ..utils.deployment.errors import BatchJobFailed, MissingScoringResults
 from ..wml_client_error import WMLClientError
-from ..workspace import WorkSpace
-from ..utils.autoai.connection import validate_source_data_connections, validate_deployment_output_connection
-
 
 if TYPE_CHECKING:
     from sklearn.pipeline import Pipeline
     from pandas import DataFrame
     from numpy import ndarray
+    from ..workspace import WorkSpace
 
 __all__ = [
     "Batch"
@@ -63,64 +75,18 @@ class Batch(BaseDeployment):
                  project_id: str = None,
                  space_id: str = None):
 
-        # note: backward compatibility
-        if wml_credentials is not None:
-            source_wml_credentials = wml_credentials
-            warn("\"wml_credentials\" parameter is depreciated, please use \"source_wml_credentials\"")
-            print("\"wml_credentials\" parameter is depreciated, please use \"source_wml_credentials\"")
-
-        if project_id is not None:
-            source_project_id = project_id
-            warn("\"project_id\" parameter is depreciated, please use \"source_project_id\"")
-            print("\"project_id\" parameter is depreciated, please use \"source_project_id\"")
-
-        if space_id is not None:
-            source_space_id = space_id
-            warn("\"space_id\" parameter is depreciated, please use \"source_space_id\"")
-            print("\"space_id\" parameter is depreciated, please use \"source_space_id\"")
-        # --- end note
-
-        # note: as workspace is not clear enough to understand, there is a possibility to use pure
-        # wml credentials with project and space IDs, but in addition we
-        # leave a possibility to use a previous WorkSpace implementation, it could be passed as a first argument
-        super().__init__(deployment_type='batch')
-        if isinstance(source_wml_credentials, WorkSpace):
-            self._source_workspace = source_wml_credentials
-
-        elif isinstance(source_wml_credentials, dict):
-            self._source_workspace = WorkSpace(wml_credentials=source_wml_credentials.copy(),
-                                               project_id=source_project_id,
-                                               space_id=source_space_id)
-        else:
-            self._source_workspace = None
-
-        if target_wml_credentials is None:
-            if isinstance(source_wml_credentials, WorkSpace):
-                self._target_workspace = source_wml_credentials
-
-            elif isinstance(source_wml_credentials, dict):
-                self._target_workspace = WorkSpace(wml_credentials=source_wml_credentials.copy(),
-                                                   project_id=source_project_id,
-                                                   space_id=source_space_id)
-            else:
-                self._target_workspace = None
-
-        else:
-            if isinstance(target_wml_credentials, WorkSpace):
-                self._target_workspace = target_wml_credentials
-
-            elif isinstance(target_wml_credentials, dict):
-                self._target_workspace = WorkSpace(wml_credentials=target_wml_credentials.copy(),
-                                                   project_id=target_project_id,
-                                                   space_id=target_space_id)
-
-                # note: only if user provides target WML information
-                if self._source_workspace is None:
-                    self._source_workspace = copy(self._target_workspace)
-
-            else:
-                self._target_workspace = None
-        # --- end note
+        super().__init__(
+            deployment_type='batch',
+            source_wml_credentials=source_wml_credentials,
+            source_project_id=source_project_id,
+            source_space_id=source_space_id,
+            target_wml_credentials=target_wml_credentials,
+            target_project_id=target_project_id,
+            target_space_id=target_space_id,
+            wml_credentials=wml_credentials,
+            project_id=project_id,
+            space_id=space_id
+        )
 
         self.name = None
         self.id = None
@@ -136,7 +102,7 @@ class Batch(BaseDeployment):
         raise NotImplementedError("Batch deployment supports only job runs.")
 
     def create(self,
-               model: Any,
+               model: str,
                deployment_name: str,
                metadata: Optional[Dict] = None,
                training_data: Optional[Union['DataFrame', 'ndarray']] = None,
@@ -147,8 +113,8 @@ class Batch(BaseDeployment):
 
         Parameters
         ----------
-        model: Union[Any, str], required
-            Model object to deploy or local path to the model.
+        model: str, required
+            AutoAI model name.
 
         deployment_name: str, required
             Name of the deployment
@@ -251,6 +217,9 @@ class Batch(BaseDeployment):
         # note: support for DataConnections and dictionaries payload
         elif isinstance(payload, list):
             if isinstance(payload[0], DataConnection):
+                if None in payload:
+                    raise NoneDataConnection('payload')
+                payload = [new_conn for conn in payload for new_conn in conn._subdivide_connection()]
                 payload = validate_source_data_connections(source_data_connections=payload,
                                                            workspace=self._source_workspace,
                                                            deployment=True)
@@ -264,9 +233,10 @@ class Batch(BaseDeployment):
                 raise ValueError("\"output_data_reference\" should be provided.")
 
             if isinstance(output_data_reference, DataConnection):
-                output_data_reference = validate_deployment_output_connection(results_data_connection=output_data_reference,
-                                                                              workspace=self._target_workspace,
-                                                                              source_data_connections = payload)
+                output_data_reference = validate_deployment_output_connection(
+                    results_data_connection=output_data_reference,
+                    workspace=self._target_workspace,
+                    source_data_connections=payload)
                 output_data_reference = output_data_reference._to_dict()
 
             scoring_payload = {
@@ -288,30 +258,19 @@ class Batch(BaseDeployment):
         ]
 
         if self._obm:
-            scoring_payload['hybrid_pipeline_hardware_specs'].append(
+            scoring_payload['hybrid_pipeline_hardware_specs'].insert(
+                0,
                 {
                     "node_runtime_id": "auto_ai.obm",
                     "hardware_spec": {
-                        "name": "M",
+                        "name": "M-Spark",
                         "num_nodes": 2
                     }
                 }
             )
 
-        # note: support for spark hummingbird
-        if self._obm:
-            change_client_get_headers(client=self._target_workspace.wml_client.training._client,
-                                      _type='new',
-                                      workspace=self._target_workspace)
-
         job_details = self._target_workspace.wml_client.deployments.create_job(self.id, scoring_payload)
 
-        # note: support for spark hummingbird
-        if self._obm:
-            change_client_get_headers(client=self._target_workspace.wml_client.training._client,
-                                      _type='old',
-                                      workspace=self._target_workspace)
-
         if background_mode:
             return job_details
 
@@ -537,7 +496,7 @@ class Batch(BaseDeployment):
         for scoring_details in resources:
             if 'scoring' in scoring_details['entity']:
                 state = scoring_details['entity']['scoring']['status']['state']
-                score_values = (scoring_details[u'metadata'][u'guid'], state,
+                score_values = (scoring_details[u'metadata'][u'id'], state,
                                 scoring_details[u'metadata'][u'created_at'],
                                 scoring_details['entity']['deployment']['id'])
                 if self.id:
@@ -553,8 +512,6 @@ class Batch(BaseDeployment):
                 pipeline_model: 'Pipeline',
                 deployment_name: str,
                 meta_props: Dict,
-                training_data: Union['DataFrame', 'ndarray'],
-                training_target: Union['DataFrame', 'ndarray'],
                 result_client=None) -> Dict:
         """
         Deploy model into WML.
@@ -567,12 +524,6 @@ class Batch(BaseDeployment):
         deployment_name: str, required
             Name of the deployment
 
-        training_data: Union['pandas.DataFrame', 'numpy.ndarray'], required
-            Training data for the model
-
-        training_target: Union['pandas.DataFrame', 'numpy.ndarray'], required
-            Target/label data for the model
-
         meta_props: dictionary, required
             Model meta properties.
 
@@ -581,9 +532,7 @@ class Batch(BaseDeployment):
         """
         deployment_details = {}
         asset_uid = self._publish_model(pipeline_model=pipeline_model,
-                                        meta_props=meta_props,
-                                        training_data=training_data,
-                                        training_target=training_target)
+                                        meta_props=meta_props)
 
         self.asset_id = asset_uid
 
@@ -606,6 +555,17 @@ class Batch(BaseDeployment):
             }
         ]
 
+        if self._obm:
+            deployment_props[
+                self._target_workspace.wml_client.deployments.ConfigurationMetaNames.HYBRID_PIPELINE_HARDWARE_SPECS
+            ].insert(0, {
+                "node_runtime_id": "auto_ai.obm",
+                "hardware_spec": {
+                    "name": "M-Spark",
+                    "num_nodes": 2
+                }
+            })
+
         print("Deploying model {} using V4 client.".format(asset_uid))
         try:
             deployment_details = self._target_workspace.wml_client.deployments.create(
