@@ -1,3 +1,16 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -20,15 +33,16 @@ class Deployments(WMLResource):
 
     """
     cloud_platform_spaces = False
+    icp_platform_spaces = False
 
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
-        if not client.ICP and not client.CLOUD_PLATFORM_SPACES:
+        if not client.ICP and not client.CLOUD_PLATFORM_SPACES and not client.ICP_PLATFORM_SPACES:
             Deployments._validate_type(client.service_instance.details, u'instance_details', dict, True)
             Deployments._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
         self.session = requests.Session()
         self._ICP = client.ICP
-        if client.CLOUD_PLATFORM_SPACES:
+        if client.CLOUD_PLATFORM_SPACES or client.ICP_PLATFORM_SPACES:
             self.ConfigurationMetaNames = DeploymentNewMetaNames()
         else:
             self.ConfigurationMetaNames = DeploymentMetaNames()
@@ -36,6 +50,9 @@ class Deployments(WMLResource):
         if client.CLOUD_PLATFORM_SPACES:
             Deployments.cloud_platform_spaces = True
 
+        if client.ICP_PLATFORM_SPACES:
+            Deployments.icp_platform_spaces = True
+
         self.ScoringMetaNames = ScoringMetaNames()
         self.DecisionOptimizationMetaNames = DecisionOptimizationMetaNames()
 
@@ -110,12 +127,15 @@ class Deployments(WMLResource):
             if meta_props is None:
                 raise WMLClientError("Invalid input. meta_props can not be empty.")
 
+            if self._client.CLOUD_PLATFORM_SPACES and 'r_shiny' in meta_props:
+                raise WMLClientError('Shiny is not supported in this release')
+
             metaProps = self.ConfigurationMetaNames._generate_resource_metadata(meta_props)
 
-            if self._client.CLOUD_PLATFORM_SPACES and 'wml_instance_id' in meta_props:
+            if (self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and 'wml_instance_id' in meta_props:
                 metaProps.update({'wml_instance_id': meta_props['wml_instance_id']})
 
-            if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+            if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 ## Remove new meataprop from older one, just in case if user provides.
                 if self.ConfigurationMetaNames.HARDWARE_SPEC in metaProps:
                     metaProps.pop(self.ConfigurationMetaNames.HARDWARE_SPEC)
@@ -159,7 +179,7 @@ class Deployments(WMLResource):
                 if rev_id is not None:
                     metaProps['asset'].update({'rev': rev_id})
 
-                if self._client.CLOUD_PLATFORM_SPACES:
+                if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                     metaProps['space_id'] = self._client.default_space_id
                 else:
                     if 'space' not in metaProps:
@@ -183,11 +203,19 @@ class Deployments(WMLResource):
                         json=metaProps,
                         headers=self._client._get_headers())
             else:
-                response = requests.post(
-                    url,
-                    json=metaProps,
-                    headers=self._client._get_headers(),
-                    verify=False)
+                if self._client.ICP_PLATFORM_SPACES:
+                    response = requests.post(
+                        url,
+                        json=metaProps,
+                        params=self._client._params(),  # version is mandatory
+                        headers=self._client._get_headers(),
+                        verify=False)
+                else:
+                    response = requests.post(
+                        url,
+                        json=metaProps,
+                        headers=self._client._get_headers(),
+                        verify=False)
 
             ## Post Deployment call executed
             if response.status_code == 202:
@@ -211,7 +239,7 @@ class Deployments(WMLResource):
                         time.sleep(5)
                         deployment_details = self._client.deployments.get_details(deployment_uid)
                         #this is wrong , needs to update for ICP
-                        if self._ICP:
+                        if self._ICP and not self._client.ICP_PLATFORM_SPACES:
                             scoringUrl = deployment_details.get(u'entity').get(u'asset').get('href').replace('https://wml-os-envoy:16600', predictionUrl)
                             deployment_details[u'entity'][u'asset']['href'] = scoringUrl
                         status = deployment_details[u'entity'][u'status'][u'state']
@@ -258,7 +286,7 @@ class Deployments(WMLResource):
         """
         Deployments._validate_type(deployment_details, u'deployment_details', dict, True)
 
-        if not Deployments.cloud_platform_spaces:
+        if not Deployments.cloud_platform_spaces and not Deployments.icp_platform_spaces:
             Deployments._validate_type_of_details(deployment_details, DEPLOYMENT_DETAILS_TYPE)
 
         try:
@@ -323,7 +351,7 @@ class Deployments(WMLResource):
 
         """
         Deployments._validate_type(deployment_details, u'deployment_details', dict, True)
-        if not Deployments.cloud_platform_spaces:
+        if not Deployments.cloud_platform_spaces and not Deployments.icp_platform_spaces:
             Deployments._validate_type_of_details(deployment_details, DEPLOYMENT_DETAILS_TYPE)
 
         try:
@@ -381,14 +409,22 @@ class Deployments(WMLResource):
 
         url = self._href_definitions.get_deployments_href()
         ## TODO:  Adding a work around till v4/deployment accepts space_id query parameter for GET
-        if self._ICP:
+        if self._ICP and not self._client.ICP_PLATFORM_SPACES:
             if deployment_uid is not None:
                 url = url + '/' + deployment_uid
-                response = requests.get(
-                    url,
-                    headers=self._client._get_headers(),
-                    verify=False
-                )
+                if self._client.ICP_35:
+                    response = requests.get(
+                        url,
+                        params = self._client._params(),
+                        headers=self._client._get_headers(),
+                        verify=False
+                    )
+                else:
+                    response = requests.get(
+                        url,
+                        headers=self._client._get_headers(),
+                        verify=False
+                    )
                 return self._handle_response(200, "deployment", response)
             else:
                 return self._get_artifact_details(url, deployment_uid, limit, 'deployments')
@@ -421,7 +457,7 @@ class Deployments(WMLResource):
 
         Deployments._validate_type(deployment_details, u'deployment', dict, True)
 
-        if not Deployments.cloud_platform_spaces:
+        if not Deployments.cloud_platform_spaces and not Deployments.icp_platform_spaces:
             Deployments._validate_type_of_details(deployment_details, DEPLOYMENT_DETAILS_TYPE)
         scoring_url = None
         try:
@@ -477,12 +513,19 @@ class Deployments(WMLResource):
                 params=self._client._params(),
                 headers=self._client._get_headers())
         else:
-            response_delete = requests.delete(
-                deployment_url,
-                #TODO:  The below line needs to uncommeted after /v4/deployment accespts query param
-               # params=self._client._params(),
-                headers=self._client._get_headers(),
-                verify=False)
+            if self._client.ICP_PLATFORM_SPACES:
+                response_delete = requests.delete(
+                    deployment_url,
+                    params=self._client._params(),
+                    headers=self._client._get_headers(),
+                    verify=False)
+            else:
+                response_delete = requests.delete(
+                    deployment_url,
+                    #TODO:  The below line needs to uncommeted after /v4/deployment accespts query param
+                   # params=self._client._params(),
+                    headers=self._client._get_headers(),
+                    verify=False)
 
         return self._handle_response(204, u'deployment deletion', response_delete, False)
 
@@ -580,34 +623,41 @@ class Deployments(WMLResource):
 
         headers = self._client._get_headers()
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             scoring_url = self._wml_credentials["url"] + "/ml/v4/deployments/" + deployment_id + "/predictions"
         else:
             scoring_url = self._wml_credentials["url"] + "/v4/deployments/" + deployment_id + "/predictions"
 
         if not self._ICP:
             if self._client.CLOUD_PLATFORM_SPACES:
-                    params = self._client._params()
-                    del params[u'space_id']
-                    response_scoring = self.session.post(
-                        scoring_url,
-                        json=payload,
-                        params=params,  # version parameter is mandatory
-                        headers=headers)
+                params = self._client._params()
+                del params[u'space_id']
+                response_scoring = self.session.post(
+                    scoring_url,
+                    json=payload,
+                    params=params,  # version parameter is mandatory
+                    headers=headers)
             else:
-
-                    response_scoring = self.session.post(
-                        scoring_url,
-                        json=payload,
-
-                        headers=headers)
+                response_scoring = self.session.post(
+                    scoring_url,
+                    json=payload,
+                    headers=headers)
         else:
-                    response_scoring = self.session.post(
-                        scoring_url,
-                        json=payload,
-                        headers=headers,
-
-                        verify=False)
+            if self._client.ICP_PLATFORM_SPACES:
+                params = self._client._params()
+                del params[u'space_id']
+                response_scoring = self.session.post(
+                    scoring_url,
+                    json=payload,
+                    params=params,  # version parameter is mandatory
+                    headers=headers,
+                    verify=False)
+            else:
+                response_scoring = self.session.post(
+                    scoring_url,
+                    json=payload,
+                    headers=headers,
+                    verify=False)
 
         return self._handle_response(200, u'scoring', response_scoring)
 
@@ -665,16 +715,18 @@ class Deployments(WMLResource):
         params = self._client._params()
         details = self.get_details(virtual_deployment_uid)
         download_url = self.get_download_url(details)
-
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_35:
+            download_url = download_url + "&version=2020-08-01"
+            params = None
         if not self._ICP:
-            if self._client.CLOUD_PLATFORM_SPACES:
-                download_url = download_url + "&version=2020-08-01"
-                response = requests.get(
-                    download_url,
-                    headers=self._client._get_headers()
-                )
-            else:
-                response = requests.get(
+            # if self._client.CLOUD_PLATFORM_SPACES:
+            #     download_url = download_url + "&version=2020-08-01"
+            #     response = requests.get(
+            #         download_url,
+            #         headers=self._client._get_headers()
+            #     )
+            # else:
+            response = requests.get(
                     download_url,
                     params=params,
                     headers=self._client._get_headers()
@@ -803,7 +855,7 @@ class Deployments(WMLResource):
             if limit is not None and index == limit:
                 break
 
-            if not self._client.CLOUD_PLATFORM_SPACES:
+            if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 if 'guid' in m[u'metadata']:
                     values.append((m[u'metadata'][u'guid'], m[u'entity'][u'name'], m[u'entity'][u'status']['state'],
                                   m[u'metadata'][u'created_at'], self._get_deployable_asset_type(m)))
@@ -820,7 +872,7 @@ class Deployments(WMLResource):
 
             index = index + 1
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             self._list(values, [u'GUID', u'NAME', u'STATE', u'CREATED', u'ARTIFACT_TYPE'], limit, 50)
         else:
             self._list(values, [u'GUID', u'NAME', u'STATE', u'CREATED'], limit, 50)
@@ -866,7 +918,7 @@ class Deployments(WMLResource):
             else:
                 state = m['entity']['decision_optimization']['status']['state']
 
-            if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 deploy_id = m['entity']['deployment']['id']
                 values.append((m[u'metadata'][u'id'], state, m[u'metadata'][u'created_at'], deploy_id))
             else:
@@ -932,8 +984,11 @@ class Deployments(WMLResource):
             print(msg)
             raise WMLClientError(msg)
 
-        if changes.get('asset') is not None and (changes.get('name') is not None or changes.get('description') is not None):
-            msg = "ASSETS update can not be combined with other fields , Please try one by one"
+        # if changes.get('asset') is not None and (changes.get('name') is not None or changes.get('description') is not None):
+        if changes.get('asset') is not None and (len(changes) > 1):
+            msg = "When ASSET is being updated/patched, other fields cannot be updated. If other fields are to be " \
+                  "updated, try without ASSET update. ASSET update triggers deployment with the new asset retaining " \
+                  "the same deployment_id"
             print(msg)
             raise WMLClientError(msg)
 
@@ -962,6 +1017,9 @@ class Deployments(WMLResource):
 
             print("Since ASSET is patched, deployment with new asset id/rev is being started. " \
                   "Monitor the status using deployments.get_details(deployment_uid) api")
+        elif 'hardware_spec' in changes:
+            updated_details = self._handle_response(202, u'deployment scaling', response)
+            ret202 = True
         else:
             updated_details = self._handle_response(200, u'deployment patch', response)
 
@@ -1014,22 +1072,21 @@ class Deployments(WMLResource):
             headers.update({'x-global-transaction-id': transaction_id})
         # making change - connection keep alive
         scoring_url = self._href_definitions.get_async_deployment_job_href()
-        if not self._ICP:
-            if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            params = self._client._params()
+        else:
+            params = None
 
-                response_scoring = self.session.post(
-                    scoring_url,
-                    json=scoring_payload,
-                    params=self._client._params(),
-                    headers=headers)
-            else:
-                response_scoring = self.session.post(
-                    scoring_url,
-                    json=scoring_payload,
-                    headers=headers)
+        if not self._ICP:
+            response_scoring = self.session.post(
+                scoring_url,
+                params=params,
+                json=scoring_payload,
+                headers=headers)
         else:
             response_scoring = self.session.post(
                 scoring_url,
+                params = params,
                 json=scoring_payload,
                 headers=headers,
                 verify=False)
@@ -1077,13 +1134,13 @@ class Deployments(WMLResource):
         flag = 0 ## To see if it is async scoring or DecisionOptimization Job
         deployment_details = self.get_details(deployment_id)
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             asset = deployment_details["entity"]["asset"]['id']
         else:
             asset = deployment_details["entity"]["asset"]["href"].split("/")[-1]
 
         do_model = False
-        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             asset_details = self._client.data_assets.get_details(asset)
             if 'wml_model' in asset_details["entity"] and 'type' in asset_details['entity']['wml_model']:
                 if 'do' in asset_details['entity']['wml_model']['type']:
@@ -1114,24 +1171,24 @@ class Deployments(WMLResource):
             for each_score_request in scoring_data:
                 lib_checker.check_lib(lib_name='pandas')
                 import pandas as pd
+                if "values" in each_score_request:
+                    scoring_values = each_score_request["values"]
+                    # Check feature types, currently supporting pandas df, numpy.ndarray, python lists and Dmatrix
+                    if (isinstance(scoring_values, pd.DataFrame)):
+                        fields_names = scoring_values.columns.values.tolist()
+                        values = scoring_values.values.tolist()
 
-                scoring_values = each_score_request["values"]
-                # Check feature types, currently supporting pandas df, numpy.ndarray, python lists and Dmatrix
-                if (isinstance(scoring_values, pd.DataFrame)):
-                    fields_names = scoring_values.columns.values.tolist()
-                    values = scoring_values.values.tolist()
-
-                    each_score_request["values"] = values
-                    if fields_names is not None:
-                        each_score_request["fields"] = fields_names
+                        each_score_request["values"] = values
+                        if fields_names is not None:
+                            each_score_request["fields"] = fields_names
 
 
-                ## If payload is a numpy dataframe
+                    ## If payload is a numpy dataframe
 
-                elif (isinstance(scoring_values, np.ndarray)):
+                    elif (isinstance(scoring_values, np.ndarray)):
 
-                    values = scoring_values.tolist()
-                    each_score_request["values"] = values
+                        values = scoring_values.tolist()
+                        each_score_request["values"] = values
 
 
                 # else:
@@ -1147,14 +1204,14 @@ class Deployments(WMLResource):
             if flag == 1:
                 payload["decision_optimization"]["input_data"] = score_payload
 
-        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             payload.update({"deployment": {"id": deployment_id}})
             if 'hardware_spec' in meta_props:
                 payload.update({"hardware_spec": meta_props[self.ConfigurationMetaNames.HARDWARE_SPEC]})
             if 'hybrid_pipeline_hardware_specs' in meta_props:
                 payload.update({"hybrid_pipeline_hardware_specs": meta_props[self.ConfigurationMetaNames.HYBRID_PIPELINE_HARDWARE_SPECS]})
 
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 payload.update({'space_id': self._client.default_space_id})
 
                 if 'name' not in payload:
@@ -1265,7 +1322,7 @@ class Deployments(WMLResource):
 
         """
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             return job_details['metadata']['id']
         else:
             return job_details['metadata']['guid']
@@ -1293,11 +1350,11 @@ class Deployments(WMLResource):
 
         """
 
-        if self._client.ICP_30:
+        if self._client.ICP_30 and not self._client.ICP_PLATFORM_SPACES:
             job_href = "/v4/deployment_jobs/" + job_details['metadata']['id']
             return job_href
         else:
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 return "/ml/v4/deployment_jobs/{}".format(job_details[u'metadata'][u'id'])
             else:
                 return job_details['metadata']['href']
@@ -1343,7 +1400,7 @@ class Deployments(WMLResource):
 
         url = self._href_definitions.get_async_deployment_jobs_href(job_uid)
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             params = self._client._params()
         else:
             params = {}
