@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from typing import TYPE_CHECKING, List, Dict
 
 from pandas import DataFrame, MultiIndex
@@ -9,7 +23,6 @@ from ibm_watson_machine_learning.utils.autoai.progress_bar import ProgressBar
 from ibm_watson_machine_learning.utils.autoai.utils import (
     fetch_pipelines, is_ipython, RunDetailsGenerator, check_dependencies_versions, try_import_autoai_libs,
     create_summary)
-from ibm_watson_machine_learning.utils.autoai.watson_studio import change_client_get_headers
 from .base_engine import BaseEngine
 
 if TYPE_CHECKING:
@@ -83,6 +96,7 @@ class WMLEngine(BaseEngine):
                             'stage_flag': True,
                             'output_logs': True,
                             'input_file_separator': self._auto_pipelines_parameters.get('csv_separator'),
+                            'encoding': self._auto_pipelines_parameters.get('encoding'),
                             'optimization': {
                                 'learning_type': self._auto_pipelines_parameters.get('prediction_type'),
                                 'label': self._auto_pipelines_parameters.get('prediction_column'),
@@ -164,6 +178,13 @@ class WMLEngine(BaseEngine):
                 'excel_sheet'] = self._auto_pipelines_parameters.get('excel_sheet')
         # --- end note
 
+        # note: KB part: send version autoai_pod_version if set in pipeline details details (See supported formats)
+        if not self._wml_client.ICP and self._auto_pipelines_parameters['autoai_pod_version'] is not None:
+            self._wml_pipeline_metadata[
+                self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = (
+                self._auto_pipelines_parameters['autoai_pod_version'])
+        # --- end note
+        
         # note: OBM part, only if user specified OBM as a preprocess method
         if self._auto_pipelines_parameters['data_join_graph']:
             # note: remove kb params for OBM only training
@@ -207,7 +228,10 @@ class WMLEngine(BaseEngine):
                                        "name": "auto_ai.obm",
                                        "app_data": {
                                            "wml_data": {
-                                               "hardware_spec": {}
+                                               "hardware_spec": {
+                                                   "name": "M-Spark",
+                                                   "num_nodes": 1
+                                               }
                                            }
                                        }
                                        }
@@ -215,27 +239,20 @@ class WMLEngine(BaseEngine):
 
             # note: in new v4 api we have missing hw_spec name
             t_size = self._auto_pipelines_parameters['data_join_graph'].t_shirt_size
-            if len(t_size) == 1:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0][
-                    'app_data']['wml_data']['hardware_spec']['name'] = t_size.upper()
-
-            else:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0][
-                    'app_data']['wml_data']['hardware_spec']['id'] = t_size
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'runtimes'][0]['app_data']['wml_data']['hardware_spec']['name'] = f"{t_size.upper()}-Spark"
             # --- end note
             # --- end note
 
             if not self._wml_client.ICP:
                 self._wml_pipeline_metadata[
-                    self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = \
-                    self._auto_pipelines_parameters['obm_pod_version']
-        # --- end note
+                    self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'runtimes'][0]['version'] = self._auto_pipelines_parameters['obm_pod_version']
 
-        # note: KB part: send version autoai_pod_version if set in pipeline details details (See supported formats)
-        if not self._wml_client.ICP and self._auto_pipelines_parameters['autoai_pod_version'] is not None:
-            self._wml_pipeline_metadata[
-                self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = \
-                self._auto_pipelines_parameters['autoai_pod_version']
+            else:
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['engine'][
+                    'template_id'] = 'spark-2.4.0-automl-icp4d-template'
         # --- end note
 
     def _initialize_wml_training_metadata(self,
@@ -332,21 +349,9 @@ class WMLEngine(BaseEngine):
         self._initialize_wml_training_metadata(training_data_reference=training_data_reference,
                                                training_results_reference=training_results_reference)
 
-        # note: support for spark hummingbird
-        if 'auto_ai.obm' in str(self._wml_pipeline_metadata):
-            change_client_get_headers(client=self._wml_client.training._client,
-                                      _type='new',
-                                      workspace=self.workspace)
-
         run_params = self._wml_client.training.run(meta_props=self._wml_training_metadata,
                                                    asynchronous=True)
 
-        # note: support for spark hummingbird
-        if 'auto_ai.obm' in str(self._wml_pipeline_metadata):
-            change_client_get_headers(client=self._wml_client.training._client,
-                                      _type='old',
-                                      workspace=self.workspace)
-
         self._current_run_id = run_params['metadata']['guid']
 
         wml_pipeline_details = self.get_params()
@@ -449,6 +454,13 @@ class WMLEngine(BaseEngine):
         else:
             return details
 
+    def cancel_run(self) -> None:
+        """Cancels an AutoAI run."""
+        if self._current_run_id is None:
+            raise FitNeeded(reason="To cancel an AutoAI run, first schedule a fit job by using the fit() method.")
+
+        self._wml_client.training.cancel(training_uid=self._current_run_id)
+
     #################################################################################
     #   WML Auto_AI Trained Pipelines Part / AutoPipelineOptimizer Pipelines Part   #
     #################################################################################
@@ -461,7 +473,7 @@ class WMLEngine(BaseEngine):
         Pandas DataFrame with computed pipelines and ML metrics.
         """
         if self._current_run_id is None:
-            raise FitNeeded(reason="To list computed pipelines, first schedule a fit job by using a fit() method.")
+            raise FitNeeded(reason="To list computed pipelines, first schedule a fit job by using the fit() method.")
 
         details = self._wml_client.training.get_details(training_uid=self._current_run_id)
         return create_summary(details=details, scoring=self._auto_pipelines_parameters['scoring'])
@@ -633,7 +645,6 @@ class WMLEngine(BaseEngine):
         Scikit-Learn pipeline.
         """
         try_import_autoai_libs()
-        check_dependencies_versions(lib_name='scikit', error=True)
         check_dependencies_versions(lib_name='numpy', error=True)
 
         run_params = self._wml_client.training.get_details(training_uid=self._current_run_id)
@@ -671,7 +682,6 @@ class WMLEngine(BaseEngine):
         Scikit-Learn pipeline.
         """
         try_import_autoai_libs()
-        check_dependencies_versions(lib_name='scikit', error=True)
         check_dependencies_versions(lib_name='numpy', error=True)
 
         best_pipeline_name = self.summary().index[0]
