@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import os
 import traceback
 import uuid
@@ -16,7 +30,7 @@ from ibm_watson_machine_learning.utils.autoai.errors import FitNeeded, MissingDa
 from ibm_watson_machine_learning.utils.autoai.local_training_message_handler import LocalTrainingMessageHandler
 from ibm_watson_machine_learning.utils.autoai.utils import (
     try_import_lale, create_summary, download_experiment_details_from_file, prepare_model_location_path,
-    download_wml_pipeline_details_from_file)
+    download_wml_pipeline_details_from_file, try_import_joblib)
 from ibm_watson_machine_learning.preprocessing import DataJoinGraph, DataJoinPipeline
 
 from .base_auto_pipelines import BaseAutoPipelines
@@ -431,7 +445,7 @@ class LocalAutoPipelines(BaseAutoPipelines):
         # note: try to download and load pipeline model from COS (auto-gen notebook scenario)
         if hasattr(self, '_result_client'):
             from ibm_watson_machine_learning.utils.autoai.utils import create_model_download_link, remove_file
-            from sklearn.externals import joblib
+            joblib = try_import_joblib()
             filename = 'pipeline_model_auto-gen_notebook.pickle'
 
             try:
@@ -597,7 +611,8 @@ class LocalAutoPipelines(BaseAutoPipelines):
             optimizer_parameters = self.get_params()
 
             if optimizer_parameters['data_join_graph']:
-                return DataJoinPipeline(self.get_preprocessed_data_connection())
+                return DataJoinPipeline(preprocessed_data_connection=self.get_preprocessed_data_connection(),
+                                        optimizer=self)
 
             else:
                 raise MissingDataPreprocessingStep(
