@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from copy import deepcopy
 from typing import TYPE_CHECKING, List, Union
 from warnings import warn
@@ -11,7 +25,8 @@ from ibm_watson_machine_learning.preprocessing import DataJoinPipeline
 from ibm_watson_machine_learning.utils.autoai.enums import (
     RunStateTypes, PipelineTypes, TShirtSize, ClassificationAlgorithms, RegressionAlgorithms, DataConnectionTypes)
 from ibm_watson_machine_learning.utils.autoai.errors import (
-    FitNotCompleted, MissingDataPreprocessingStep, WrongDataJoinGraphNodeName, DataSourceSizeNotSupported, TrainingDataSourceIsNotFile)
+    FitNotCompleted, MissingDataPreprocessingStep, WrongDataJoinGraphNodeName, DataSourceSizeNotSupported,
+    TrainingDataSourceIsNotFile, NoneDataConnection)
 from ibm_watson_machine_learning.utils.autoai.utils import try_import_lale
 from ibm_watson_machine_learning.utils.autoai.wsd_ui import (
     save_computed_pipelines_for_ui, save_experiment_for_ui, save_metadata_for_ui)
@@ -74,6 +89,9 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         Name or number of the excel sheet to use. Only use when xlsx file is an input.
         Default is 0.
 
+    encoding: str, optional
+            Encoding type for CSV training file.
+
     positive_label: str, optional
             The positive class to report when binary classification.
             When multiclass or regression, this will be ignored.
@@ -105,6 +123,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
                  data_join_graph: 'DataJoinGraph' = None,
                  csv_separator: Union[List[str], str] = ',',
                  excel_sheet: Union[str, int] = 0,
+                 encoding: str = 'utf-8',
                  positive_label: str = None,
                  data_join_only: bool = False,
                  notebooks=False,
@@ -124,6 +143,7 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             'data_join_graph': data_join_graph or False,
             'csv_separator': csv_separator,
             'excel_sheet': excel_sheet,
+            'encoding': encoding,
             'positive_label': positive_label,
             'data_join_only': data_join_only,
             'notebooks': notebooks,
@@ -242,6 +262,11 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         >>>     background_mode=False)
         """
 
+        if None in training_data_reference:
+            raise NoneDataConnection('training_data_reference')
+
+        training_data_reference = [new_conn for conn in training_data_reference for new_conn in conn._subdivide_connection()]
+
         # note: check if DataJoinGraph node names are correct and equivalent
         # to training DataConnections IDs/data_join_node_names
         if self.params['data_join_graph']:
@@ -370,6 +395,10 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         """
         return self._engine.get_run_details()
 
+    def cancel_run(self) -> None:
+        """Cancels an AutoAI run."""
+        self._engine.cancel_run()
+
     #################################
     #   Pipeline models operations  #
     #################################
@@ -484,7 +513,25 @@ class RemoteAutoPipelines(BaseAutoPipelines):
 
         elif astype == PipelineTypes.LALE:
             from lale.helpers import import_from_sklearn_pipeline
-            return import_from_sklearn_pipeline(pipeline_model)
+            # note: join preprocessing step to final pipeline (enables wider visualization and pretty print)
+            try:
+                import numpy as np
+                preprocessing_object = self.get_preprocessing_pipeline()
+                preprocessing_pipeline = preprocessing_object.lale_pipeline
+                
+                # note: fake fit for preprocessing pipeline to be able to predict
+                preprocessing_pipeline = preprocessing_pipeline.fit(np.array([[1, 2, 3], [2, 3, 4]]), np.array([1, 2]))
+                return preprocessing_pipeline >> import_from_sklearn_pipeline(pipeline_model)
+
+            except MissingDataPreprocessingStep:
+                return import_from_sklearn_pipeline(pipeline_model)
+
+            except BaseException as e:
+                message = f"Cannot load preprocessing step. Error: {e}"
+                print(message)
+                warn(message)
+                return import_from_sklearn_pipeline(pipeline_model)
+            # --- end note
 
         else:
             raise ValueError('Incorrect value of \'astype\'. '
@@ -614,7 +661,8 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         optimizer_parameters = self.get_params()
 
         if optimizer_parameters['data_join_graph']:
-            return DataJoinPipeline(self.get_preprocessed_data_connection(), self)
+            return DataJoinPipeline(preprocessed_data_connection=self.get_preprocessed_data_connection(),
+                                    optimizer=self)
 
         else:
             raise MissingDataPreprocessingStep(
