@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 __all__ = [
     "AutoPipelinesRuns"
 ]
@@ -175,6 +189,8 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
                 'parameters'].get('input_file_separator', ',')
             excel_sheet = optimizer_config['entity']['document']['pipelines'][0]['nodes'][kb_node_number][
                 'parameters'].get('excel_sheet', 0)
+            encoding = optimizer_config['entity']['document']['pipelines'][0]['nodes'][kb_node_number][
+                'parameters'].get('encoding', 'utf-8')
 
             params = {
                 'name': name,
@@ -188,7 +204,8 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
                 'daub_include_only_estimators': kb_parameters.get('daub_include_only_estimators'),
                 'train_sample_rows_test_size': kb_parameters.get('train_sample_rows_test_size'),
                 'csv_separator': csv_separator,
-                'excel_sheet': excel_sheet
+                'excel_sheet': excel_sheet,
+                'encoding': encoding
             }
 
             if kb_parameters.get('train_sample_rows_test_size'):
@@ -211,7 +228,9 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
 
             obm_wml_data = optimizer_config['entity']['document']['runtimes'][obm_runtime_number]['app_data'][
                 'wml_data']
-            data_join_graph.t_shirt_size = obm_wml_data['hardware_spec']['name']
+            # need to trim X-spark t-shirt name only to first letter
+            t_name = obm_wml_data['hardware_spec']['name']
+            data_join_graph.t_shirt_size = t_name if len(t_name) == 1 else t_name[0]
 
             params['data_join_graph'] = data_join_graph
         # --- end note
@@ -295,9 +314,15 @@ class AutoPipelinesRuns(BaseAutoPipelinesRuns):
             training_result_reference = metadata.get('training_result_reference')
             run_id = training_result_reference.location.path.split('/')[-3]
 
-            return AutoAI(self._workspace.wml_credentials,
-                          space_id=self._workspace.space_id
-                          ).runs.get_optimizer(run_id)
+            # note: CP4D notebook scenario
+            if self._wml_engine is not None:
+                return AutoAI(self._workspace).runs.get_optimizer(run_id)
+
+            # note WSD
+            else:
+                return AutoAI(self._workspace.wml_credentials,
+                              space_id=self._workspace.space_id
+                              ).runs.get_optimizer(run_id)
         # --- end note
 
     def get_data_connections(self, run_id: str) -> List['DataConnection']:
