@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -89,7 +94,7 @@ class Experiments(WMLResource):
 
         metaProps = self.ConfigurationMetaNames._generate_resource_metadata(meta_props)
         ##Check if default space is set
-        if self._client.CAMS or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CAMS and not self._client.ICP_PLATFORM_SPACES:
             if self._client.default_space_id is not None:
                 metaProps['space'] = {'href': "/v4/spaces/"+self._client.default_space_id}
             elif self._client.default_project_id is not None:
@@ -97,7 +102,7 @@ class Experiments(WMLResource):
             else:
                 raise WMLClientError("It is mandatory to set the space/project id. Use client.set.default_space(<SPACE_UID>)/client.set.default_project(<PROJECT_UID>) to proceed.")
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             if self._client.default_space_id is not None:
                 metaProps['space_id'] = self._client.default_space_id
             elif self._client.default_project_id is not None:
@@ -125,12 +130,21 @@ class Experiments(WMLResource):
                     headers=self._client._get_headers()
                 )
         else:
-            response_experiment_post = requests.post(
-                self._href_definitions.get_experiments_href(),
-                json=metaProps,
-                headers=self._client._get_headers(),
-                verify=False
-            )
+            if self._client.ICP_PLATFORM_SPACES:
+                response_experiment_post = requests.post(
+                    self._href_definitions.get_experiments_href(),
+                    json=metaProps,
+                    params=self._client._params(),
+                    headers=self._client._get_headers(),
+                    verify=False
+                )
+            else:
+                response_experiment_post = requests.post(
+                    self._href_definitions.get_experiments_href(),
+                    json=metaProps,
+                    headers=self._client._get_headers(),
+                    verify=False
+                )
 
         return self._handle_response(201, u'saving experiment', response_experiment_post)
 
@@ -388,7 +402,7 @@ class Experiments(WMLResource):
         self._client._check_if_either_is_set()
         experiment_resources = self.get_details(limit=limit)[u'resources']
         header_list = [u'GUID', u'NAME', u'CREATED']
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             experiment_values = [(m[u'metadata'][u'id'], m[u'metadata'][u'name'], m[u'metadata'][u'created_at']) for m in
                                  experiment_resources]
             header_list = [u'ID', u'NAME', u'CREATED']
@@ -415,7 +429,7 @@ class Experiments(WMLResource):
         experiment_id = str_type_conv(experiment_id)
         Experiments._validate_type(experiment_id, u'experiment_id', STR_TYPE, True)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
         else:
@@ -445,7 +459,7 @@ class Experiments(WMLResource):
         model_uid = str_type_conv(experiment_uid)
         Experiments._validate_type(experiment_uid, u'experiment_uid', STR_TYPE, True)
         Experiments._validate_type(rev_uid, u'rev_uid', int, True)
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
         else:
@@ -475,7 +489,7 @@ class Experiments(WMLResource):
 
         Experiments._validate_type(experiment_uid, u'experiment_uid', STR_TYPE, True)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
         else:
