@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -28,13 +33,13 @@ class Functions(WMLResource):
 
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
-        if not client.ICP and not client.WSD and not client.CLOUD_PLATFORM_SPACES:
+        if not client.ICP and not client.WSD and not client.CLOUD_PLATFORM_SPACES and not client.ICP_PLATFORM_SPACES:
             Functions._validate_type(client.service_instance.details, u'instance_details', dict, True)
             Functions._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
         self._ICP = client.ICP
         self._WSD = client.WSD
 
-        if client.CLOUD_PLATFORM_SPACES:
+        if client.CLOUD_PLATFORM_SPACES or client.ICP_PLATFORM_SPACES:
             self.ConfigurationMetaNames = FunctionNewMetaNames()
         else:
             self.ConfigurationMetaNames = FunctionMetaNames()
@@ -233,7 +238,7 @@ score = {}()
                 #     data = datafile.read()
                 details = Functions._wsd_create_asset(self, "wml_function", payload, meta_props, content_path,user_content_file)
             else:
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                     if self.ConfigurationMetaNames.RUNTIME_UID not in meta_props and self.ConfigurationMetaNames.SOFTWARE_SPEC_UID not in meta_props:
                         import sys
                         # print('No RUNTIME_UID passed. Creating default runtime... ', end="")
@@ -272,7 +277,7 @@ score = {}()
                         meta_props[self.ConfigurationMetaNames.RUNTIME_UID] = runtime_uid
 
                 function_metadata = self.ConfigurationMetaNames._generate_resource_metadata(meta_props, with_validation=True,client=self._client)
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                     # For Cloud convergence, FunctionNewMetaNames is used which has path defined
                     # So, _generate_resource_metadata above will properly fill in the relevant fields
                     input_schemas = []
@@ -291,7 +296,7 @@ score = {}()
                             "output": output_schemas}
                         })
 
-                if self._client.CAMS:
+                if self._client.CAMS and not self._client.ICP_PLATFORM_SPACES:
                     if self._client.default_space_id is not None:
                         function_metadata['space'] = {'href': "/v4/spaces/" + self._client.default_space_id}
                     elif self._client.default_project_id is not None:
@@ -300,7 +305,7 @@ score = {}()
                         raise WMLClientError(
                             "It is mandatory to set the space/project id. Use client.set.default_space(<SPACE_UID>)/client.set.default_project(<PROJECT_UID>) to proceed.")
 
-                if self._client.CLOUD_PLATFORM_SPACES:
+                if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                     if self._client.default_space_id is not None:
                         function_metadata['space_id'] = self._client.default_space_id
                     elif self._client.default_project_id is not None:
@@ -321,17 +326,24 @@ score = {}()
                                                       json=function_metadata,
                                                       headers=self._client._get_headers())
                 else:
-                    response_post = requests.post(self._href_definitions.get_functions_href(),
-                                                  json=function_metadata,
-                                                  headers=self._client._get_headers(),
-                                                  verify=False)
+                    if self._client.ICP_PLATFORM_SPACES:
+                        response_post = requests.post(self._href_definitions.get_functions_href(),
+                                                      json=function_metadata,
+                                                      params=self._client._params(),
+                                                      headers=self._client._get_headers(),
+                                                      verify=False)
+                    else:
+                        response_post = requests.post(self._href_definitions.get_functions_href(),
+                                                      json=function_metadata,
+                                                      headers=self._client._get_headers(),
+                                                      verify=False)
 
                 details = self._handle_response(expected_status_code=201,
                                                 operationName=u'saving function',
                                                 response=response_post)
                 ##TODO_V4 Take care of this since the latest swagger endpoint is not working
 
-                if self._client.CLOUD_PLATFORM_SPACES:
+                if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                     function_content_url = self._href_definitions.get_function_href(
                         details['metadata']['id']) + "/code"
                 else:
@@ -345,7 +357,11 @@ score = {}()
                                                                params=self._client._params(),
                                                                headers=put_header)
                     else:
-                        response_definition_put = requests.put(function_content_url, data=data, params = self._client._params(),headers=put_header, verify=False)
+                        response_definition_put = requests.put(function_content_url,
+                                                               data=data,
+                                                               params = self._client._params(),
+                                                               headers=put_header,
+                                                               verify=False)
 
         except Exception as e:
             raise e
@@ -356,7 +372,7 @@ score = {}()
                 pass
 
         if self._client.WSD is None:
-            if not self._client.CLOUD_PLATFORM_SPACES:
+            if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 if response_definition_put.status_code != 200:
                    self._delete(details['metadata']['guid'])
                 self._handle_response(200, u'saving function content', response_definition_put,json_response=False)
@@ -390,7 +406,7 @@ score = {}()
         """
         if self._client.WSD:
             raise WMLClientError(u'Updating Function is not supported for IBM Watson Studio Desktop. ')
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and update_function is not None:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES and update_function is not None:
             raise WMLClientError(u'Updating Function content not supported for this release')
 
         self._client._check_if_either_is_set()
@@ -413,8 +429,9 @@ score = {}()
             response = requests.patch(url, json=patch_payload,params = self._client._params(), headers=headers, verify=False)
         updated_details = self._handle_response(200, u'function patch', response)
 
-        if (self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES) and update_function is not None:
+        if (self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and update_function is not None:
             self._update_function_content(function_uid,  update_function)
+
         return updated_details
 
     def _update_function_content(self, function_uid, updated_function):
@@ -496,7 +513,7 @@ score = {}()
                     pass
                 raise WMLClientError('Exception during getting function code.', e)
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             function_content_url = self._href_definitions.get_function_href(function_uid) + "/code"
         else:
             function_content_url = self._href_definitions.get_function_href(function_uid) + "/content"
@@ -546,7 +563,7 @@ score = {}()
 
         if os.path.isfile(filename):
             raise WMLClientError(u'File with name: \'{}\' already exists.'.format(filename))
-        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not applicable for this release')
 
         artifact_uid = str_type_conv(function_uid)
@@ -567,7 +584,7 @@ score = {}()
             artifact_content_url = self._href_definitions.get_wsd_model_attachment_href() + \
                                    urllib.parse.quote('wml_function/' + attachment_url, safe='')
         else:
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 artifact_content_url = self._href_definitions.get_function_href(artifact_uid) + '/code'
             else:
                 artifact_content_url = self._href_definitions.get_function_latest_revision_content_href(artifact_uid)
@@ -584,12 +601,21 @@ score = {}()
                                      headers=self._client._get_headers(),
                                      stream=True)
                 else:
-                    r = requests.get(artifact_content_url, params= self._client._params(), headers=self._client._get_headers(), stream=True)
+                    r = requests.get(artifact_content_url,
+                                     params= self._client._params(),
+                                     headers=self._client._get_headers(),
+                                     stream=True)
             else:
                 params = self._client._params()
+
                 if rev_uid is not None:
                     params.update({'revision_id': rev_uid})
-                r = requests.get(artifact_content_url, params=params,headers=self._client._get_headers(), stream=True, verify=False)
+
+                r = requests.get(artifact_content_url,
+                                 params=params,
+                                 headers=self._client._get_headers(),
+                                 stream=True,
+                                 verify=False)
             if r.status_code != 200:
                 raise ApiRequestFailure(u'Failure during {}.'.format("downloading function"), r)
 
@@ -863,7 +889,7 @@ score = {}()
             self._client._check_if_either_is_set()
             function_resources = self.get_details()[u'resources']
 
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 function_values = [(m[u'metadata'][u'id'],
                                     m[u'metadata'][u'name'],
                                     m[u'metadata'][u'created_at'],
@@ -919,8 +945,10 @@ score = {}()
                     * Space guid is mandatory for move action\n
 
         """
-        if self._client.WSD:
-            raise WMLClientError(u'Clone method is not supported for IBM Watson Studio Desktop. ')
+        # if self._client.WSD or self._client.ICP_PLATFORM_SPACES or :
+        #     raise WMLClientError(u'Clone method is not supported for IBM Watson Studio Desktop. ')
+
+        raise WMLClientError("Not supported")
 
         artifact = str_type_conv(function_uid)
         Functions._validate_type(artifact, 'function_uid', STR_TYPE, True)
@@ -963,7 +991,7 @@ score = {}()
         function_uid = str_type_conv(function_uid)
         Functions._validate_type(function_uid, u'pipeline_uid', STR_TYPE, False)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 u'Revision support is not there in this release')
         else:
@@ -992,7 +1020,7 @@ score = {}()
         Functions._validate_type(function_uid, u'function_uid', STR_TYPE, True)
         Functions._validate_type(rev_uid, u'rev_uid', int, True)
 
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 'Revision APIs are not supported in this release. ')
         else:
@@ -1021,14 +1049,14 @@ score = {}()
 
         Functions._validate_type(function_uid, u'function_uid', STR_TYPE, True)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Revision support is not there in this release.')
         else:
             url = self._href_definitions.get_function_href(function_uid)
 
             # CP4D logic is wrong. By passing "revisions" in second param above for _get_artifact_details()
             # it won't even consider limit value and also GUID gives only rev number, not actual guid
-            if not self._client.CLOUD_PLATFORM_SPACES:
+            if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 function_resources = self._get_artifact_details(url, "revisions", limit, 'function revisions')[u'resources']
                 function_values = [
                     (m[u'metadata'][u'rev'], m[u'metadata'][u'name'], m[u'metadata'][u'created_at']) for m in
