@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 __all__ = [
     "DataConnection",
     "S3Connection",
@@ -14,6 +28,7 @@ __all__ = [
 import io
 import os
 import uuid
+import copy
 from copy import deepcopy
 from typing import Union, Tuple, List, TYPE_CHECKING, Optional
 from warnings import warn
@@ -26,7 +41,7 @@ import requests
 from ibm_watson_machine_learning.utils.autoai.enums import PredictionType, DataConnectionTypes
 from ibm_watson_machine_learning.utils.autoai.errors import (
     MissingAutoPipelinesParameters, UseWMLClient, MissingCOSStudioConnection, MissingProjectLib,
-    HoldoutSplitNotSupported, InvalidCOSCredentials, MissingLocalAsset
+    HoldoutSplitNotSupported, InvalidCOSCredentials, MissingLocalAsset, InvalidIdType
 )
 from ibm_watson_machine_learning.utils.autoai.watson_studio import get_project
 from ibm_watson_machine_learning.wml_client_error import MissingValue, ApiRequestFailure
@@ -49,6 +64,11 @@ class DataConnection(BaseDataConnection):
 
     location: Union[S3Location, FSLocation, CP4DAssetLocation, WMLSAssetLocation, CloudAssetLocation, WSDAssetLocation],
         required location parameters of specific type
+
+    data_join_node_name: Union[str, List[str]], optional
+        Names for node(s). If no value provided, data file name will be used as node name. If str will be passed,
+        it will became node name. If multiple names will be passed, several nodes will have the same data connection
+        (used for excel files with multiple sheets).
     """
 
     def __init__(self,
@@ -60,7 +80,7 @@ class DataConnection(BaseDataConnection):
                                  'CloudAssetLocation',
                                  'DeploymentOutputAssetLocation'],
                  connection: Optional['S3Connection'] = None,
-                 data_join_node_name: str = None):
+                 data_join_node_name: Union[str, List[str]] = None):
         super().__init__()
 
         if isinstance(connection, S3Connection):
@@ -147,6 +167,17 @@ class DataConnection(BaseDataConnection):
                     "Please create a COS connection from the UI and insert "
                     "the cell with project API connection (Insert project token)")
 
+    def _subdivide_connection(self):
+        if type(self.id) is str or not self.id:
+            return [self]
+        else:
+            def cpy(new_id):
+                child = copy.copy(self)
+                child.id = new_id
+                return child
+
+            return [cpy(id) for id in self.id]
+
     def _to_dict(self) -> dict:
         """
         Convert DataConnection object to dictionary representation.
@@ -155,6 +186,10 @@ class DataConnection(BaseDataConnection):
         -------
         Dictionary
         """
+
+        if self.id and type(self.id) is list:
+            raise InvalidIdType(list)
+
         _dict = {"type": self.type}
 
         # note: for OBM (id of DataConnection if an OBM node name)
@@ -210,7 +245,7 @@ class DataConnection(BaseDataConnection):
 
         else:
             # note: recognizing environment by location href
-            if 'projet_id=' in _dict['location']['href']:
+            if 'project_id=' in _dict['location']['href']:
                 asset_location = CP4DAssetLocation._set_path(href=_dict['location']['href'])
             else:
                 asset_location = WMLSAssetLocation._set_path(href=_dict['location']['href'])
@@ -227,7 +262,8 @@ class DataConnection(BaseDataConnection):
     def read(self,
              with_holdout_split: bool = False,
              csv_separator: str = ',',
-             excel_sheet: Union[str, int] = 0) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
+             excel_sheet: Union[str, int] = 0,
+             encoding: Optional[str] = 'utf-8') -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
         """
         Download dataset stored in remote data storage.
 
@@ -242,6 +278,9 @@ class DataConnection(BaseDataConnection):
         excel_sheet: Union[str, int], optional
             Excel file sheet name to use, default is 0.
 
+        encoding: str, optional
+            Encoding type of the CSV
+
         Returns
         -------
         pandas.DataFrame contains dataset from remote data storage or Tuple[pandas.DataFrame, pandas.DataFrame]
@@ -262,10 +301,12 @@ class DataConnection(BaseDataConnection):
                        f"if you are using historical runs, just call historical_optimizer.get_data_connections()")
 
         # note: allow to read data at any time
-        elif (('csv_separator' not in self.auto_pipeline_params and 'excel_sheet' not in self.auto_pipeline_params)
-              or csv_separator != ',' or excel_sheet != 0):
+        elif (('csv_separator' not in self.auto_pipeline_params and 'excel_sheet' not in self.auto_pipeline_params and
+               'encoding' not in self.auto_pipeline_params)
+              or csv_separator != ',' or excel_sheet != 0 or encoding != 'utf-8'):
             self.auto_pipeline_params['csv_separator'] = csv_separator
             self.auto_pipeline_params['excel_sheet'] = excel_sheet
+            self.auto_pipeline_params['encoding'] = encoding
         # --- end note
 
         data = DataFrame()
