@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 import re
 
@@ -121,6 +126,9 @@ PROJECT = u'{}/v2/projects/{}'
 V4GA_CLOUD_MIGRATION = u'{}/ml/v4/repository'
 V4GA_CLOUD_MIGRATION_ID = u'{}/ml/v4/repository/{}'
 
+REMOTE_TRAINING_SYSTEM = u'{}/v4/remote_training_systems'
+REMOTE_TRAINING_SYSTEM_ID = u'{}/v4/remote_training_systems/{}'
+
 def is_url(s):
     res = re.match('https?:\/\/.+', s)
     return res is not None
@@ -132,13 +140,14 @@ def is_uid(s):
 
 
 class HrefDefinitions:
-    def __init__(self, wml_credentials, cloud_platform_spaces=False, platform_url=None, cams_url=None):
+    def __init__(self, wml_credentials, cloud_platform_spaces=False, platform_url=None, cams_url=None, cp4d_platform_spaces=False):
         self._wml_credentials = wml_credentials
         self.cloud_platform_spaces = cloud_platform_spaces
+        self.cp4d_platform_spaces = cp4d_platform_spaces
         self.platform_url = platform_url
         self.cams_url = cams_url
 
-        if self.cloud_platform_spaces:
+        if self.cloud_platform_spaces or self.cp4d_platform_spaces:
             self.prepend = '/ml'
         else:
             self.prepend = ''
@@ -260,10 +269,16 @@ class HrefDefinitions:
             return SPACES_PLATFORM_HREF_PATTERN.format(self.platform_url)
 
     def get_platform_spaces_member_href(self, spaces_id, member_id):
-        return SPACES_MEMBER_HREF_PATTERN.format(self.platform_url, spaces_id, member_id)
+        if self.platform_url is None:
+            return SPACES_MEMBER_HREF_PATTERN.format(self._wml_credentials['url'], spaces_id, member_id)
+        else:
+            return SPACES_MEMBER_HREF_PATTERN.format(self.platform_url, spaces_id, member_id)
 
     def get_platform_spaces_members_href(self,spaces_id):
-        return SPACES_MEMBERS_HREF_PATTERN.format(self.platform_url, spaces_id)
+        if self.platform_url is None:
+            return SPACES_MEMBERS_HREF_PATTERN.format(self._wml_credentials['url'], spaces_id)
+        else:
+            return SPACES_MEMBERS_HREF_PATTERN.format(self.platform_url, spaces_id)
 
     def get_v4_instance_id_href(self):
         return V4_INSTANCE_ID_HREF_PATTERN.format(self._wml_credentials['url'],
@@ -465,6 +480,25 @@ class HrefDefinitions:
     def v4ga_cloud_migration_id_href(self, migration_id):
         return V4GA_CLOUD_MIGRATION_ID.format(self._wml_credentials['url'], migration_id)
 
+    def remote_training_system_href(self, remote_training_systems_id):
+        if self.platform_url is None:
+            return REMOTE_TRAINING_SYSTEM_ID.format(self._wml_credentials['url'], remote_training_systems_id)
+        else:
+            return REMOTE_TRAINING_SYSTEM_ID.format(self.platform_url, remote_training_systems_id)
+
+    def remote_training_systems_href(self):
+        # return PACKAGE_EXTENSIONS.format(self._wml_credentials['url'])
+        if self.platform_url is None:
+            return REMOTE_TRAINING_SYSTEM.format(self._wml_credentials['url'])
+        else:
+            return REMOTE_TRAINING_SYSTEM.format(self.platform_url)
+
+    def remote_training_systems_href(self):
+        return REMOTE_TRAINING_SYSTEM.format(self._wml_credentials['url'] + self.prepend)
+
+    def remote_training_system_href(self, remote_training_systems_id):
+        return REMOTE_TRAINING_SYSTEM_ID.format(self._wml_credentials['url'] + self.prepend, remote_training_systems_id)
+
     # def get_envs_href(self):
     #     return DEPLOYMENT_ENVS_HREF_PATTERN.format(self._wml_credentials['url'])
     #
