@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import requests
 import json
 import base64
@@ -17,11 +31,20 @@ class ServiceInstanceNewPlan:
         self._client = client
         self._wml_credentials = client.wml_credentials
 
+        if self._client.ICP_PLATFORM_SPACES:
+            if self.get_instance_id() == 'openshift' or self.get_instance_id() == 'wml_local':
+                self._wml_credentials[u'url'] = self.get_url()
+            else:
+                self._wml_credentials[u'url'] = self.get_url() + ':31843'
+            # TODO: Check if this is used anywhere.. from initial searches, doesn't seem like
+            # self._wml_credentials[u'instance_id'] = "999"
+
         # This is used in connections.py
         self._href_definitions = HrefDefinitions(self._wml_credentials,
                                                  self._client.CLOUD_PLATFORM_SPACES,
                                                  self._client.PLATFORM_URL,
-                                                 self._client.CAMS_URL
+                                                 self._client.CAMS_URL,
+                                                 self._client.ICP_PLATFORM_SPACES,
                                                  )
         self._client.wml_token = self._create_token()
         self._logger.info(u'Successfully prepared token: ' + self._client.wml_token)
@@ -76,6 +99,46 @@ class ServiceInstanceNewPlan:
         """
         return self._wml_credentials['url']
 
+    def get_username(self):
+        """
+             Get username for your Watson Machine Learning service. Applicable only for Cloud Pak for Data
+
+             **Output**
+
+             .. important::
+
+                **returns**: username\n
+                **return type**: str
+
+             **Example**
+
+             >>> instance_details = client.service_instance.get_username()
+        """
+        if self._client.ICP_PLATFORM_SPACES:
+            return self._wml_credentials['username']
+        else:
+            raise WMLClientError("Not applicable for Cloud")
+
+    def get_password(self):
+        """
+             Get password for your Watson Machine Learning service. Applicable only for Cloud Pak for Data
+
+             **Output**
+
+             .. important::
+
+                **returns**: password\n
+                **return type**: str
+
+             **Example**
+
+             >>> instance_details = client.service_instance.get_password()
+        """
+        if self._client.ICP_PLATFORM_SPACES:
+            return self._wml_credentials['password']
+        else:
+            raise WMLClientError("Not applicable for Cloud")
+
     def get_details(self):
         """
              Get information about your Watson Machine Learning instance.
@@ -92,36 +155,39 @@ class ServiceInstanceNewPlan:
              >>> instance_details = client.service_instance.get_details()
         """
 
-        if self._wml_credentials is not None:
+        if not self._client.ICP:
+            if self._wml_credentials is not None:
 
-            if self._wml_credentials['instance_id'] == 'invalid':
-                raise WMLClientError('instance_id for this plan is picked up from the space with which '
-                                     'this instance_id is associated with. Set the space with associated '
-                                     'instance_id to be able to use this function')
+                if self._wml_credentials['instance_id'] == 'invalid':
+                    raise WMLClientError('instance_id for this plan is picked up from the space with which '
+                                         'this instance_id is associated with. Set the space with associated '
+                                         'instance_id to be able to use this function')
 
-                # /ml/v4/instances will need either space_id or project_id as mandatory params
-            # We will enable this service instance class only during create space or
-            # set space/project. So, space_id/project_id would have been populated at this point
-            headers = self._client._get_headers()
+                    # /ml/v4/instances will need either space_id or project_id as mandatory params
+                # We will enable this service instance class only during create space or
+                # set space/project. So, space_id/project_id would have been populated at this point
+                headers = self._client._get_headers()
 
-            del headers[u'X-WML-User-Client']
-            if 'ML-Instance-ID' in headers:
-                headers.pop('ML-Instance-ID')
-            headers.pop(u'x-wml-internal-switch-to-new-v4')
-            params = {'version': self._client.version_param}
-            response_get_instance = requests.get(
-                self._href_definitions.get_v4_instance_id_href(),
-                # params=self._client._params(),
-                params={'version': self._client.version_param},
-                headers=headers
-            )
+                del headers[u'X-WML-User-Client']
+                if 'ML-Instance-ID' in headers:
+                    headers.pop('ML-Instance-ID')
+                headers.pop(u'x-wml-internal-switch-to-new-v4')
+                # params = {'version': self._client.version_param}
+                response_get_instance = requests.get(
+                    self._href_definitions.get_v4_instance_id_href(),
+                    # params=self._client._params(),
+                    params={'version': self._client.version_param},
+                    headers=headers
+                )
 
-            if response_get_instance.status_code == 200:
-                return response_get_instance.json()
+                if response_get_instance.status_code == 200:
+                    return response_get_instance.json()
+                else:
+                    raise ApiRequestFailure(u'Getting instance details failed.', response_get_instance)
             else:
-                raise ApiRequestFailure(u'Getting instance details failed.', response_get_instance)
+                raise NoWMLCredentialsProvided
         else:
-            raise NoWMLCredentialsProvided
+            return {}
 
 
     def _get_token(self):
@@ -132,6 +198,9 @@ class ServiceInstanceNewPlan:
             if self._get_expiration_datetime() - timedelta(minutes=50) < datetime.now():
                 self._client.wml_token = self._get_IAM_token()
                 self._client.repository._refresh_repo_client()
+            elif self._get_expiration_datetime() - timedelta(minutes=30) < datetime.now():
+                self._client.repository._refresh_repo_client()
+                self._refresh_token()
 
         return self._client.wml_token
 
@@ -139,10 +208,34 @@ class ServiceInstanceNewPlan:
 
         if self._client.proceed is True:
             return self._wml_credentials["token"]
-        if self._client._is_IAM():
-            return self._get_IAM_token()
+
+        if not self._client.ICP_PLATFORM_SPACES:
+            if self._client._is_IAM():
+                return self._get_IAM_token()
+            else:
+                raise WMLClientError('apikey for IAM token is not provided in credentials for the client.')
         else:
-            raise WMLClientError('apikey for IAM token is not provided in credentials for the client.')
+            token_url = self._wml_credentials['url'].replace(':31002', ':31843') + '/v1/preauth/validateAuth'
+            response = requests.get(token_url, auth=(self.get_username(), self.get_password()), verify=False)
+
+            if response.status_code == 200:
+                token = response.json().get(u'accessToken')
+            else:
+                raise WMLClientError(u'Error during getting token for the given credentials.', response)
+            return token
+
+    def _refresh_token(self):
+        import requests
+        if self._client.proceed is True:
+            self._client.wml_token = self._wml_credentials["token"]
+
+        token_url = self._wml_credentials['url'].replace(':31002', ':31843') + '/v1/preauth/validateAuth'
+        response = requests.get(token_url, auth=(self.get_username(), self.get_password()), verify=False)
+
+        if response.status_code == 200:
+            self._client.wml_token = response.json().get(u'accessToken')
+        else:
+            raise WMLClientError(u'Error during refreshing the token.', response)
 
     def _get_expiration_datetime(self):
         token_parts = self._client.wml_token.split('.')
@@ -178,5 +271,5 @@ class ServiceInstanceNewPlan:
         if response.status_code == 200:
             token = response.json().get(u'access_token')
         else:
-            raise ApiRequestFailure(u'Error during getting IAM Token.', response)
+            raise WMLClientError(u'Error during getting IAM Token.', response)
         return token
\ No newline at end of file
