@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -110,7 +115,7 @@ class Pipelines(WMLResource):
                   }
                 ]
               }
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_35:
             if self.ConfigurationMetaNames.COMMAND in meta_props:
                 doc["pipelines"][0]["nodes"][0]["parameters"]["command"] = meta_props[self.ConfigurationMetaNames.COMMAND]
             if self.ConfigurationMetaNames.RUNTIMES in meta_props:
@@ -193,7 +198,7 @@ class Pipelines(WMLResource):
             else:
                 raise WMLClientError("It is mandatory to set the space/project id. Use client.set.default_space(<SPACE_UID>)/client.set.default_project(<PROJECT_UID>) to proceed.")
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_35:
             if self._client.default_space_id is not None:
                 pipeline_meta['space_id'] = self._client.default_space_id
             elif self._client.default_project_id is not None:
@@ -268,6 +273,14 @@ class Pipelines(WMLResource):
                 pipeline_details = self._handle_response(201, u'creating new pipeline', creation_response)
                 return WMLResource._wsd_get_required_element_from_response(self, pipeline_details)
 
+            elif self._client.ICP_35:
+                    creation_response = requests.post(
+                        self._href_definitions.get_pipelines_href(),
+                        headers=self._client._get_headers(),
+                        params=self._client._params(),
+                        json=pipeline_meta,
+                        verify=False
+                     )
             else:
                 creation_response = requests.post(
                     self._wml_credentials['url'] + '/v4/pipelines',
@@ -295,7 +308,7 @@ class Pipelines(WMLResource):
         pipeline_uid = str_type_conv(pipeline_uid)
         Pipelines._validate_type(pipeline_uid, u'pipeline_uid', STR_TYPE, False)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_35:
             raise WMLClientError(
                 u'Revision support is not there in this version WML server. It is supported only from 3.1.0 onwards.')
         else:
@@ -463,7 +476,7 @@ class Pipelines(WMLResource):
         pipeline_uid = str_type_conv(pipeline_uid)
         Pipelines._validate_type(pipeline_uid, u'pipeline_uid', STR_TYPE, True)
         Pipelines._validate_type(rev_uid, u'rev_uid', int, True)
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_35:
             raise WMLClientError(
                 'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
         else:
@@ -600,7 +613,7 @@ class Pipelines(WMLResource):
             self._client._check_if_either_is_set()
 
             pipeline_resources = self.get_details(limit=limit)[u'resources']
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_35:
                 pipeline_values = [(m[u'metadata'][u'id'], m[u'metadata'][u'name'], m[u'metadata'][u'created_at']) for m
                                    in pipeline_resources]
 
@@ -634,7 +647,7 @@ class Pipelines(WMLResource):
 
         Pipelines._validate_type(pipeline_uid, u'pipeline_uid', STR_TYPE, True)
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_35:
             raise WMLClientError(
                 u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
         else:
@@ -687,7 +700,7 @@ class Pipelines(WMLResource):
                     * Space id is mandatory for move action\n
 
         """
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_35:
             raise WMLClientError(" Clone pipeline operation is not supported. ")
         artifact = str_type_conv(pipeline_uid)
         Pipelines._validate_type(artifact, 'exp_id', STR_TYPE, True)
