@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests,json, time
@@ -30,6 +35,7 @@ class PlatformSpaces(WMLResource):
 
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
+        self._client = client
 
     def _get_resources(self, url, op_name, params=None):
         if params is not None and 'limit' in params.keys():
@@ -39,11 +45,19 @@ class PlatformSpaces(WMLResource):
                 raise WMLClientError('Limit cannot be larger than 1000.')
 
         if len(params) > 0:
-            response_get = requests.get(
-                url,
-                headers=self._client._get_headers(),
-                params=params
-            )
+            if not self._ICP:
+                response_get = requests.get(
+                    url,
+                    headers=self._client._get_headers(),
+                    params=params
+                )
+            else:
+                response_get = requests.get(
+                    url,
+                    headers=self._client._get_headers(),
+                    params=params,
+                    verify=False
+                )
 
             return self._handle_response(200, op_name, response_get)
         else:
@@ -51,11 +65,10 @@ class PlatformSpaces(WMLResource):
             resources = []
 
             while True:
-                response_get = requests.get(
-                    url,
-                    headers=self._client._get_headers(),
-
-                )
+                if not self._ICP:
+                    response_get = requests.get( url, headers=self._client._get_headers())
+                else:
+                    response_get = requests.get( url, headers=self._client._get_headers(), verify=False)
 
                 result = self._handle_response(200, op_name, response_get)
                 resources.extend(result['resources'])
@@ -63,7 +76,6 @@ class PlatformSpaces(WMLResource):
                 if 'next' not in result:
                     break
                 else:
-
                     url = self._wml_credentials["url"]+result['next']['href']
                     if('start=invalid' in url):
                         break
@@ -74,7 +86,8 @@ class PlatformSpaces(WMLResource):
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def store(self, meta_props, background_mode=True):
         """
-                Create a space. The instance associated with the space will be used for billing purposes
+                Create a space. The instance associated with the space via COMPUTE will be used for billing purposes on
+                cloud. Note that STORAGE and COMPUTE are applicable only for cloud
 
                 **Parameters**
 
@@ -110,7 +123,14 @@ class PlatformSpaces(WMLResource):
         # TODO add error handling and cleaning (remove the file)
         PlatformSpaces._validate_type(meta_props, u'meta_props', dict, True)
 
-        if 'compute' in meta_props:
+        if ('compute' in meta_props or 'storage' in meta_props) and self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError("'STORAGE' and 'COMPUTE' meta props are not applicable on "
+                                 "Cloud Pak for Data. If using any of these, remove and retry")
+
+        if 'storage' not in meta_props and self._client.CLOUD_PLATFORM_SPACES:
+            raise WMLClientError("'STORAGE' is mandatory for cloud")
+
+        if 'compute' in meta_props and self._client.CLOUD_PLATFORM_SPACES:
             if 'name' not in meta_props[u'compute'] or 'crn' not in meta_props[u'compute']:
                 raise WMLClientError("'name' and 'crn' is mandatory for 'COMPUTE'")
             temp_meta = meta_props[u'compute']
@@ -125,16 +145,22 @@ class PlatformSpaces(WMLResource):
 
         )
 
-        if 'compute' in meta_props:
+        if 'compute' in meta_props and self._client.CLOUD_PLATFORM_SPACES:
             payload_compute = []
             payload_compute.append(space_meta[u'compute'])
             space_meta[u'compute'] = payload_compute
 
-        creation_response = requests.post(
+        if not self._ICP:
+            creation_response = requests.post(
                 self._href_definitions.get_platform_spaces_href(),
                 headers=self._client._get_headers(),
-                json=space_meta
-        )
+                json=space_meta)
+        else:
+            creation_response = requests.post(
+                self._href_definitions.get_platform_spaces_href(),
+                headers=self._client._get_headers(),
+                json=space_meta,
+                verify=False)
 
         spaces_details = self._handle_response(202, u'creating new spaces', creation_response)
 
@@ -142,7 +168,7 @@ class PlatformSpaces(WMLResource):
         # during client.set.default_space since that's where space is associated with client
         # and also in client.set.default_project
         #
-        if 'compute' in spaces_details['entity'].keys():
+        if 'compute' in spaces_details['entity'].keys() and self._client.CLOUD_PLATFORM_SPACES:
             instance_id = spaces_details['entity']['compute'][0]['guid']
             self._client.wml_credentials[u'instance_id'] = instance_id
             self._client.service_instance = ServiceInstanceNewPlan(self._client)
@@ -232,11 +258,17 @@ class PlatformSpaces(WMLResource):
         PlatformSpaces._validate_type(space_id, u'space_id', STR_TYPE, True)
 
         space_endpoint = self._href_definitions.get_platform_space_href(space_id)
-        response_delete = requests.delete(space_endpoint, headers=self._client._get_headers())
+
+        if not self._ICP:
+            response_delete = requests.delete(space_endpoint, headers=self._client._get_headers())
+        else:
+            response_delete = requests.delete(space_endpoint, headers=self._client._get_headers(), verify=False)
+
+        response = self._handle_response(202, u'space deletion', response_delete, False)
 
         print('DELETED')
 
-        return self._handle_response(202, u'space deletion', response_delete, False)
+        return response
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_details(self, space_id=None, limit=None):
@@ -269,10 +301,10 @@ class PlatformSpaces(WMLResource):
         href = self._href_definitions.get_platform_space_href(space_id)
 
         if space_id is not None:
-            response_get = requests.get(
-                href,
-                headers=self._client._get_headers()
-            )
+            if not self._ICP:
+                response_get = requests.get(href, headers=self._client._get_headers())
+            else:
+                response_get = requests.get(href, headers=self._client._get_headers(), verify=False)
 
             return self._handle_response(200, 'Get space', response_get)
 
@@ -345,6 +377,7 @@ class PlatformSpaces(WMLResource):
     def update(self, space_id, changes):
         """
                 Updates existing space metadata. 'STORAGE' cannot be updated
+                STORAGE and COMPUTE are applicable only for cloud
 
                 **Parameters**
 
@@ -371,6 +404,10 @@ class PlatformSpaces(WMLResource):
                  >>> space_details = client.spaces.update(space_id, changes=metadata)
         """
 
+        if ('compute' in changes or 'storage' in changes) and self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError("'STORAGE' and 'COMPUTE' meta props are not applicable on"
+                                 "Cloud Pak for Data. If using any of these, remove and retry")
+
         if 'storage' in changes:
             raise WMLClientError("STORAGE cannot be updated")
 
@@ -381,7 +418,7 @@ class PlatformSpaces(WMLResource):
 
         details = self.get_details(space_id)
 
-        if 'compute' in changes:
+        if 'compute' in changes and self._client.CLOUD_PLATFORM_SPACES:
             changes[u'compute'][u'type'] = 'machine_learning'
 
             payload_compute = []
@@ -395,12 +432,16 @@ class PlatformSpaces(WMLResource):
         print("patch payload: ", patch_payload)
 
         href = self._href_definitions.get_platform_space_href(space_id)
-        response = requests.patch(href, json=patch_payload, headers=self._client._get_headers())
+
+        if not self._ICP:
+            response = requests.patch(href, json=patch_payload, headers=self._client._get_headers())
+        else:
+            response = requests.patch(href, json=patch_payload, headers=self._client._get_headers(), verify=False)
 
         updated_details = self._handle_response(200, u'spaces patch', response)
 
         # Cloud Convergence
-        if 'compute' in updated_details['entity'].keys():
+        if 'compute' in updated_details['entity'].keys() and self._client.CLOUD_PLATFORM_SPACES:
             instance_id = updated_details['entity']['compute'][0]['guid']
             self._client.wml_credentials[u'instance_id'] = instance_id
             self._client.service_instance = ServiceInstanceNewPlan(self._client)
@@ -465,11 +506,17 @@ class PlatformSpaces(WMLResource):
             client=self._client
         )
 
-        creation_response = requests.post(
-            self._href_definitions.get_platform_spaces_members_href(space_id),
-            headers=self._client._get_headers(),
-            json=space_meta
-        )
+        if not self._ICP:
+            creation_response = requests.post(
+                self._href_definitions.get_platform_spaces_members_href(space_id),
+                headers=self._client._get_headers(),
+                json=space_meta)
+        else:
+            creation_response = requests.post(
+                self._href_definitions.get_platform_spaces_members_href(space_id),
+                headers=self._client._get_headers(),
+                json=space_meta,
+                verify=False)
 
         # TODO: Change response code one they change it to 201
         members_details = self._handle_response(200, u'creating new members', creation_response)
@@ -506,13 +553,12 @@ class PlatformSpaces(WMLResource):
 
         href = self._href_definitions.get_platform_spaces_member_href(space_id, member_id)
 
-        response_get = requests.get(
-            href,
-            headers=self._client._get_headers()
-        )
-        return self._handle_response(200, 'Get space member', response_get)
+        if not self._ICP:
+            response_get = requests.get(href, headers=self._client._get_headers())
+        else:
+            response_get = requests.get(href, headers=self._client._get_headers(), verify=False)
 
-        return self._get_no_space_artifact_details(href, member_id, None, 'space members')
+        return self._handle_response(200, 'Get space member', response_get)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def delete_member(self, space_id, member_id):
@@ -546,7 +592,10 @@ class PlatformSpaces(WMLResource):
 
         member_endpoint = self._href_definitions.get_platform_spaces_member_href(space_id, member_id)
 
-        response_delete = requests.delete(member_endpoint, headers=self._client._get_headers())
+        if not self._ICP:
+            response_delete = requests.delete(member_endpoint, headers=self._client._get_headers())
+        else:
+            response_delete = requests.delete(member_endpoint, headers=self._client._get_headers(), verify=False)
 
         print('DELETED')
 
@@ -637,7 +686,12 @@ class PlatformSpaces(WMLResource):
              # patch_payload = self.MemberMetaNames._generate_patch_payload(details, changes, with_validation=True)
 
             href = self._href_definitions.get_platform_spaces_member_href(space_id,member_id)
-            response = requests.patch(href, json=patch_payload, headers=self._client._get_headers())
+
+            if not self._ICP:
+                response = requests.patch(href, json=patch_payload, headers=self._client._get_headers())
+            else:
+                response = requests.patch(href, json=patch_payload, headers=self._client._get_headers(), verify=False)
+
             updated_details = self._handle_response(200, u'members patch', response)
 
             return updated_details
