@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -39,26 +44,25 @@ class Repository(WMLResource):
            >>> client.repository.FunctionMetaNames.show()
         #. To view PipelineMetaNames, use: \n 
            >>> client.repository.PipelineMetaNames.show()
-        #. To view SpacesMetaNames, use: \n 
-           >>> client.repository.SpacesMetaNames.show()
-        #. To view MemberMetaNames, use: \n
-           >>> client.repository.MemberMetaNames.show()
+
     """
 
     cloud_platform_spaces = False
+    icp_platform_spaces = False
 
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
-        if not client.ICP and not client.WSD and not client.CLOUD_PLATFORM_SPACES:
+        if not client.ICP and not client.WSD and not client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             Repository._validate_type(client.service_instance.details, u'instance_details', dict, True)
             Repository._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
         self._ICP = client.ICP
         self._WSD = client.WSD
         self._ml_repository_client = None
         Repository.cloud_platform_spaces = client.CLOUD_PLATFORM_SPACES
+        Repository.icp_platform_spaces = client.ICP_PLATFORM_SPACES
 
         self.ExperimentMetaNames = ExperimentMetaNames()
-        if not client.CLOUD_PLATFORM_SPACES:
+        if not client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             self.FunctionMetaNames = FunctionMetaNames()
         else:
             self.FunctionMetaNames = FunctionNewMetaNames()
@@ -79,23 +83,33 @@ class Repository(WMLResource):
                 if self._client.project_id is not None:
                     self._ml_repository_client._add_header('X-Watson-Project-ID', self._client.project_id)
             else:
+                if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+                    platform_spaces = True
+                else:
+                    platform_spaces = False
+
                 self._ml_repository_client.authorize_with_iamtoken(self._client.wml_token,
                                                                    self._wml_credentials[u'instance_id'],
-                                                                   self._client.CLOUD_PLATFORM_SPACES)
+                                                                   platform_spaces)
                 self._ml_repository_client._add_header('X-WML-User-Client', 'PythonClient')
                 # Cloud Convergence
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                    self._ml_repository_client._add_header('ML-Instance-ID', self._wml_credentials[u'instance_id'])
                 if self._client.project_id is not None:
                     self._ml_repository_client._add_header('X-Watson-Project-ID', self._client.project_id)
         else:
             if self._client._is_IAM():
+                if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+                    platform_spaces = True
+                else:
+                    platform_spaces = False
+
                 self._ml_repository_client.authorize_with_iamtoken(self._client.wml_token,
                                                                    self._wml_credentials[u'instance_id'],
-                                                                   self._client.CLOUD_PLATFORM_SPACES)
+                                                                   platform_spaces)
                 self._ml_repository_client._add_header('X-WML-User-Client', 'PythonClient')
                 # Cloud Convergence
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                    self._ml_repository_client._add_header('ML-Instance-ID', self._wml_credentials[u'instance_id'])
                 if self._client.project_id is not None:
                     self._ml_repository_client._add_header('X-Watson-Project-ID', self._client.project_id)
@@ -184,8 +198,8 @@ class Repository(WMLResource):
         """
 
 
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
-            raise WMLClientError(u'Not supported in this release.')
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
         return self._client.spaces.store(meta_props)
 
 
@@ -223,8 +237,8 @@ class Repository(WMLResource):
                  >>> }
                  >>> members_details = client.repository.create_member(space_uid=space_id, meta_props=metadata)
         """
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
-            raise WMLClientError(u'Not supported in this release.')
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
         return self._client.spaces.create_member(space_uid,meta_props)
 
     @staticmethod
@@ -250,7 +264,8 @@ class Repository(WMLResource):
                 .. important::
 
                    #. **meta_props**:  meta data of the pipeline configuration. To see available meta names use:\n
-                                    >>> client.pipelines.ConfigurationMetaNames.get()
+
+                        >>> client.pipelines.ConfigurationMetaNames.get()
 
                       **type**: dict\n
 
@@ -319,8 +334,7 @@ class Repository(WMLResource):
                     #. **label_column_names**:  Label column names of the trained Scikit-Learn/XGBoost models.\n
                        **type**: numpy.ndarray and list\n
 
-                    #. **subtrainingId**:  The subtraining ID for a training created via an experiment.\n
-                       **type**: str\n
+
 
                 **Output**
 
@@ -333,9 +347,6 @@ class Repository(WMLResource):
 
                     * For a keras model, model content is expected to contain a .h5 file and an archived version of it.\n
 
-                    * For deploying a keras model, it is mandatory to pass the FRAMEWORK_LIBRARIES along with other metaprops.\n
-                      >>> client.repository.ModelMetaNames.FRAMEWORK_LIBRARIES: [{'name':'keras', 'version': '2.1.3'}]
-
                     * feature_names is an optional argument containing the feature names for the training data in case of Scikit-Learn/XGBoost models. Valid types are numpy.ndarray and list. This is applicable only in the case where the training data is not of type - pandas.DataFrame.\n
 
                     * If the training data is of type pandas.DataFrame and feature_names are provided, feature_names are ignored.\n
@@ -348,17 +359,21 @@ class Repository(WMLResource):
 
                     In more complicated cases you should create proper metadata, similar to this one:\n
 
+                    >>> sw_spec_id = client.software_specifications.get_id_by_name('scikit-learn_0.22-py3.6')
+                    >>> sw_spec_id
                     >>> metadata = {
                     >>>        client.repository.ModelMetaNames.NAME: 'customer satisfaction prediction model',
-                    >>>        client.repository.ModelMetaNames.TYPE: 'tensorflow_1.5',
-                    >>>        client.repository.ModelMetaNames.RUNTIME_UID: 'tensorflow_1.5-py3'
+                    >>>        client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_id,
+                    >>>        client.repository.ModelMetaNames.TYPE: 'scikit-learn_0.22'
                     >>>}
 
                     In case when you want to provide input data schema of the model, you can provide it as part of meta
 
+                    >>> sw_spec_id = client.software_specifications.get_id_by_name('spss-modeler_18.1')
+                    >>> sw_spec_id
                     >>> metadata = {
                     >>>        client.repository.ModelMetaNames.NAME: 'customer satisfaction prediction model',
-                    >>>        client.repository.ModelMetaNames.RUNTIME_UID: 'spss-modeler_18.1',
+                    >>>        client.repository.ModelMetaNames.SOFTWARE_SPEC_UID: sw_spec_id,
                     >>>        client.repository.ModelMetaNames.TYPE: 'spss-modeler_18.1',
                     >>>        client.repository.ModelMetaNames.INPUT_DATA_SCHEMA: [{'id': 'test',
                     >>>                                                             'type': 'list',
@@ -433,7 +448,7 @@ class Repository(WMLResource):
                 * Space guid is mandatory for move action\n
 
             """
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError('Cloning is not supported.')
         artifact = str_type_conv(artifact_id)
         Models._validate_type(artifact, 'artifact_id', STR_TYPE, True)
@@ -536,7 +551,7 @@ class Repository(WMLResource):
                 >>> metadata = {
                 >>>    client.repository.FunctionMetaNames.NAME: "function",
                 >>>    client.repository.FunctionMetaNames.DESCRIPTION: "This is ai function",
-                >>>    client.repository.FunctionMetaNames.RUNTIME_UID: "53dc4cf1-252f-424b-b52d-5cdd9814987f",
+                >>>    client.repository.FunctionMetaNames.SOFTWARE_SPEC_UID: "53dc4cf1-252f-424b-b52d-5cdd9814987f",
                 >>>    client.repository.FunctionMetaNames.INPUT_DATA_SCHEMAS: [{"fields": [{"metadata": {}, "type": "string", "name": "GENDER", "nullable": True}]}],
                 >>>    client.repository.FunctionMetaNames.OUTPUT_DATA_SCHEMAS:[{"fields": [{"metadata": {}, "type": "string", "name": "GENDER", "nullable": True}]}],
                 >>>    client.repository.FunctionMetaNames.TAGS: [{"value": "ProjectA", "description": "Functions created for ProjectA"}]
@@ -791,8 +806,8 @@ class Repository(WMLResource):
                  >>> }
                  >>> space_details = client.repository.update_space(space_uid, changes=metadata)
         """
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
-            raise WMLClientError('Not supported in this release.')
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
         return self._client.spaces.update(space_uid, changes)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
@@ -861,9 +876,9 @@ class Repository(WMLResource):
             return self._client._models.download(artifact_uid, filename, rev_uid,format)
         elif res['function'] is True:
             return self._client._functions.download(artifact_uid, filename, rev_uid)
-        elif not self._client.CLOUD_PLATFORM_SPACES and res['library'] is True:
+        elif not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_35 and res['library'] is True:
             return self._client.runtimes.download_library(artifact_uid, filename)
-        elif not self._client.CLOUD_PLATFORM_SPACES and res['runtime'] is True:
+        elif not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_35 and res['runtime'] is True:
             return self._client.runtimes.download_configuration(artifact_uid, filename)
         else:
             raise WMLClientError('Unexpected type of artifact to download or Artifact with artifact_uid: \'{}\' does not exist.'.format(artifact_uid) )
@@ -892,28 +907,22 @@ class Repository(WMLResource):
 
         artifact_uid = str_type_conv(artifact_uid)
         Repository._validate_type(artifact_uid, u'artifact_uid', STR_TYPE, True)
+        if (self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and self._if_deployment_exist_for_asset(artifact_uid):
+            raise WMLClientError(
+                u'Cannot delete artifact that has existing deployments. Please delete all associated deployments and try again')
+        params = self._client._params()
+        if Repository.cloud_platform_spaces or self._client.ICP_PLATFORM_SPACES:
+            # ideally purge_on_delete=true query param has to be provided for deletion of cams assets
+            # This doesn't seem to be done for CP4D 3.0.1 and before. We should do this for CP4D 3.5
+            params.update({'purge_on_delete': 'true'})
 
         if not self._ICP:
-            if self._client.CLOUD_PLATFORM_SPACES and self._if_deployment_exist_for_asset(artifact_uid):
-                    raise WMLClientError(
-                            u'Cannot delete artifact that has existing deployments. Please delete all associated deployments and try again')
-            if Repository.cloud_platform_spaces:
-                # ideally purge_on_delete=true query param has to be provided for deletion of cams assets
-                # This doesn't seem to be done for CP4D 3.0.1 and before. We should do this for CP4D 3.5
-
-                params = self._client._params()
-                params.update({'purge_on_delete':'true'})
-
-                response = requests.delete(self._href_definitions.get_asset_href(artifact_uid),
+            response = requests.delete(self._href_definitions.get_asset_href(artifact_uid),
                                            params=params,
                                            headers=self._client._get_headers())
-            else:
-                response = requests.delete(self._href_definitions.get_asset_href(artifact_uid),
-                                           params=self._client._params(),
-                                           headers=self._client._get_headers())
         else:
             response = requests.delete(self._href_definitions.get_asset_href(artifact_uid),
-                                       params=self._client._params(),
+                                       params=params,
                                        headers=self._client._get_headers(), verify=False)
 
         if response.status_code == 200 or response.status_code == 204:
@@ -924,7 +933,7 @@ class Repository(WMLResource):
                 response = self._handle_response(204, u'delete assets', response)
                 return response
         else:
-            if Repository.cloud_platform_spaces:
+            if Repository.cloud_platform_spaces or self._client.ICP_PLATFORM_SPACES:
                 # Since we are using /v2/assets for deletion, don't need all the logic
                 # in the following else block. The else block is applicable only for cloud beta
                 # and has to be kept till then. For 3.5, move logic to same as cloud convergence
@@ -999,7 +1008,7 @@ class Repository(WMLResource):
                 pipeline_details = self.get_pipeline_details()
                 function_details = self._client._functions.get_details()
 
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                     space_details = self._client.spaces.get_details()
                     library_details = self._client.runtimes.get_library_details()
                     runtime_details = self._client.runtimes.get_details()
@@ -1043,11 +1052,11 @@ class Repository(WMLResource):
                     details = self.get_pipeline_details(artifact_uid)
                 elif uid_type[u'function'] is True:
                     details = self._client._functions.get_details(artifact_uid)
-                elif not self._client.CLOUD_PLATFORM_SPACES and uid_type[u'runtime'] is True:
+                elif not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES and uid_type[u'runtime'] is True:
                     details = self._client.runtimes.get_details(artifact_uid)
-                elif not self._client.CLOUD_PLATFORM_SPACES and uid_type[u'library'] is True:
+                elif not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES and uid_type[u'library'] is True:
                     details = self._client.runtimes.get_library_details(artifact_uid)
-                elif not self._client.CLOUD_PLATFORM_SPACES and uid_type[u'space'] is True:
+                elif not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES and uid_type[u'space'] is True:
                     details = self._client.spaces.get_details(artifact_uid)
                 else:
                     raise WMLClientError(u'Getting artifact details failed. Artifact uid: \'{}\' not found.'.format(artifact_uid))
@@ -1109,7 +1118,7 @@ class Repository(WMLResource):
 
         """
 
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError('Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
         return self._client._models.get_revision_details(model_uid, rev_uid)
 
@@ -1177,7 +1186,7 @@ class Repository(WMLResource):
 
         """
 
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
 
@@ -1244,7 +1253,7 @@ class Repository(WMLResource):
 
         """
 
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError('Not supported in this release')
         return self._client._functions.get_revision_details(function_uid, rev_id)
 
@@ -1305,7 +1314,7 @@ class Repository(WMLResource):
                 >>> pipeline_rev_details = client.repository.get_pipeline_revision_details(pipeline_uid, rev_id)
         """
 
-        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.ICP_30 and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(
                 'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
         return self._client.pipelines.get_revision_details(pipeline_uid, rev_id)
@@ -1340,8 +1349,8 @@ class Repository(WMLResource):
             >>> space_details = client.repository.get_space_details()
         """
 
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
-            raise WMLClientError('Not supported in this release.')
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
         Repository._validate_type(space_uid, u'space_uid', STR_TYPE, False)
         Repository._validate_type(limit, u'limit', int, False)
         return self._client.spaces.get_details(space_uid, limit)
@@ -1374,8 +1383,8 @@ class Repository(WMLResource):
             >>> member_details = client.repository.get_member_details(space_uid,member_id)
         """
 
-        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES:
-            raise WMLClientError('Not supported in this release.')
+        if self._client.WSD or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
         return self._client.spaces.get_members_details(space_uid,member_id)
 
 
@@ -1722,8 +1731,8 @@ class Repository(WMLResource):
         if 'WSD_PLATFORM' in os.environ and os.environ['WSD_PLATFORM'] == 'True':
             raise WMLClientError(u'Spaces APIs are not supported for Watson Studio Desktop.')
 
-        if Repository.cloud_platform_spaces:
-            raise WMLClientError('Not supported in this release')
+        if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
         return Spaces.get_uid(space_details)
 
@@ -1752,8 +1761,8 @@ class Repository(WMLResource):
         if 'WSD_PLATFORM' in os.environ and os.environ['WSD_PLATFORM'] == 'True':
             raise WMLClientError(u'Spaces APIs are not supported for Watson Studio Desktop.')
 
-        if Repository.cloud_platform_spaces:
-            raise WMLClientError('Not supported in this release')
+        if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
         return Spaces.get_member_uid(member_details)
 
@@ -1783,8 +1792,8 @@ class Repository(WMLResource):
         if 'WSD_PLATFORM' in os.environ and os.environ['WSD_PLATFORM'] == 'True':
             raise WMLClientError(u'Spaces APIs are not supported for Watson Studio Desktop.')
 
-        if Repository.cloud_platform_spaces:
-            raise WMLClientError('Not supported in this release')
+        if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
         return Spaces.get_href(space_details)
 
@@ -1814,8 +1823,8 @@ class Repository(WMLResource):
         if 'WSD_PLATFORM' in os.environ and os.environ['WSD_PLATFORM'] == 'True':
             raise WMLClientError(u'Spaces APIs are not supported for Watson Studio Desktop.')
 
-        if Repository.cloud_platform_spaces:
-            raise WMLClientError('Not supported in this release')
+        if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
         return Spaces.get_member_href(member_details)
 
@@ -1855,7 +1864,7 @@ class Repository(WMLResource):
                 u'Use list method of specific artifact.')
 
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             endpoints = {
                 u'model': self._href_definitions.get_published_models_href(),
                 u'experiment': self._href_definitions.get_experiments_href(),
@@ -1899,7 +1908,7 @@ class Repository(WMLResource):
 
         pool.close()
 
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             model_values = [(m[u'metadata'][u'id'], m[u'metadata'][u'name'], m[u'metadata'][u'created_at'],
                              m[u'entity'][u'type'], u'model') for m in resources[u'model']]
             experiment_values = [
@@ -1999,8 +2008,8 @@ class Repository(WMLResource):
         if self._client.WSD:
             raise WMLClientError('list_spaces - Listing spaces is not supported for Watson Studio Desktop.')
 
-        if Repository.cloud_platform_spaces:
-            raise WMLClientError('Not supported in this release')
+        if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+            raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
         self._client.spaces.list(limit=limit)
 
@@ -2072,8 +2081,8 @@ class Repository(WMLResource):
             if self._client.WSD:
                 raise WMLClientError('list_members - Listing members is not supported for Watson Studio Desktop.')
 
-            if Repository.cloud_platform_spaces:
-                raise WMLClientError('Not supported in this release')
+            if Repository.cloud_platform_spaces or Repository.icp_platform_spaces:
+                raise WMLClientError(u"Not supported in this release. Use methods in 'client.spaces' instead")
 
             self._client.spaces.list_members(space_uid=space_uid,limit=limit)
 
@@ -2106,7 +2115,7 @@ class Repository(WMLResource):
             return artifact_type_exists
 
         else:
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 endpoints = {
                     u'model': self._href_definitions.get_model_last_version_href(artifact_uid),
                     u'pipeline': self._href_definitions.get_pipeline_href(artifact_uid),
