@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2020
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -64,7 +69,7 @@ class Script(WMLResource):
         if response.status_code == 200:
             response = self._get_required_element_from_response(self._handle_response(200, u'get asset details', response))
 
-            if not self._client.CLOUD_PLATFORM_SPACES:
+            if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 return response
             else:
 
@@ -127,9 +132,11 @@ class Script(WMLResource):
             client=self._client
         )
 
-        response = self._create_asset(script_meta, file_path)
+        name, extension = os.path.splitext(file_path)
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        response = self._create_asset(script_meta, file_path, extension)
+
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
 
@@ -147,7 +154,7 @@ class Script(WMLResource):
 
             return final_response
 
-    def _create_asset(self, script_meta, file_path):
+    def _create_asset(self, script_meta, file_path, extension='.py'):
 
         ##Step1: Create a data asset
         f = {'file': open(file_path, 'rb')}
@@ -188,6 +195,13 @@ class Script(WMLResource):
             "version": 1
         }
 
+        if extension == '.py':
+            lang = 'python3'
+        elif extension == '.R':
+            lang = 'R'
+        else:
+            lang = 'python3'
+
         #check if the software spec specified is base or derived and
         # accordingly update the entity for asset creation
         # TODO WSD2.0
@@ -197,7 +211,7 @@ class Script(WMLResource):
             sw_spec_details = self._client.software_specifications.get_details(script_meta[u'software_spec_uid'])
 
             if(sw_spec_details[u'entity'][u'software_specification'][u'type'] == 'base'):
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                     asset_meta = {
                         "metadata": {
                             "name": name,
@@ -209,8 +223,9 @@ class Script(WMLResource):
                         "entity": {
                             "script": {
                                 "language": {
-                                    "name": "python3"
-                                },
+                                    # "name": "python3"
+                                    "name": lang
+                    },
                                 "software_spec": {
                                     "base_id": script_meta[u'software_spec_uid']
                                 }
@@ -230,8 +245,9 @@ class Script(WMLResource):
                              "script": {
                                  "ml_version": "4.0.0",
                                  "language": {
-                                     "name": "python3"
-                                 },
+                                     # "name": "python3"
+                                     "name": lang
+                     },
                                  "software_spec": {
                                      "base_id": script_meta[u'software_spec_uid']
                                  }
@@ -239,7 +255,7 @@ class Script(WMLResource):
                          }
                      }
             else:
-                if not self._client.CLOUD_PLATFORM_SPACES:
+                if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                     asset_meta = {
                         "metadata": {
                             "name": name,
@@ -251,8 +267,9 @@ class Script(WMLResource):
                         "entity": {
                             "script": {
                                 "language": {
-                                    "name": "python3"
-                                },
+                                    # "name": "python3"
+                                    "name": lang
+                    },
                                 "software_spec": {
                                     "id": script_meta[u'software_spec_uid']
                                 }
@@ -272,7 +289,8 @@ class Script(WMLResource):
                             "script": {
                                 "ml_version": "4.0.0",
                                 "language": {
-                                    "name": "python3"
+                                    # "name": "python3"
+                                    "name": lang
                                 },
                                 "software_spec": {
                                     "id": script_meta[u'software_spec_uid']
@@ -292,8 +310,9 @@ class Script(WMLResource):
                 "entity": {
                     "script": {
                         "language": {
-                            "name": "python3"
-                        },
+                            # "name": "python3"
+                            "name": lang
+            },
                         "software_spec": {
                             "id": script_meta[u'software_spec_uid']
                         }
@@ -323,14 +342,7 @@ class Script(WMLResource):
                         json=asset_meta
                 )
             else:
-                asset_type_response = requests.post(
-                    self._wml_credentials['url'] + "/v2/asset_types?",
-                    headers=self._client._get_headers(),
-                    json=base_script_asset,
-                    params=self._client._params(),
-                    verify=False
-                )
-                if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
+                if self._client.ICP_PLATFORM_SPACES:
                     creation_response = requests.post(
                         self._href_definitions.get_data_assets_href(),
                         headers=self._client._get_headers(),
@@ -338,6 +350,22 @@ class Script(WMLResource):
                         params=self._client._params(),
                         verify=False
                     )
+                else:
+                    asset_type_response = requests.post(
+                        self._wml_credentials['url'] + "/v2/asset_types?",
+                        headers=self._client._get_headers(),
+                        json=base_script_asset,
+                        params=self._client._params(),
+                        verify=False
+                    )
+                    if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
+                        creation_response = requests.post(
+                            self._href_definitions.get_data_assets_href(),
+                            headers=self._client._get_headers(),
+                            json=asset_meta,
+                            params=self._client._params(),
+                            verify=False
+                        )
 
             asset_details = self._handle_response(201, u'creating new asset', creation_response)
             #Step2: Create attachment
@@ -486,7 +514,7 @@ class Script(WMLResource):
              >>> client.script.download(asset_uid,"script_file.zip")
 
          """
-        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not applicable for this release')
 
         Script._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
@@ -669,7 +697,7 @@ class Script(WMLResource):
         """
 
         # We need to enable this once we add functionality for WSD
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         Script._validate_type(script_uid, 'script_uid', STR_TYPE, True)
@@ -830,7 +858,7 @@ class Script(WMLResource):
 
         response = self._get_required_element_from_response(self._handle_response(200, "Get script details", response))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             entity = response[u'entity']
@@ -879,7 +907,7 @@ class Script(WMLResource):
         Script._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
 
         if not self._ICP:
-            if self._client.CLOUD_PLATFORM_SPACES and self._if_deployment_exist_for_asset(asset_uid):
+            if (self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and self._if_deployment_exist_for_asset(asset_uid):
                 raise WMLClientError(
                     u'Cannot delete script that has existing deployments. Please delete all associated deployments and try again')
 
@@ -909,7 +937,7 @@ class Script(WMLResource):
            >>> script_revision = client.scripts.create_revision(script_uid)
         """
         # We need to enable this once we add functionality for WSD
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -925,7 +953,7 @@ class Script(WMLResource):
         response = self._get_required_element_from_response(
             self._create_revision_artifact_for_assets(script_uid, 'Script'))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             entity = response[u'entity']
@@ -959,7 +987,7 @@ class Script(WMLResource):
            >>> client.scripts.list_revisions(script_uid)
         """
         # We need to enable this once we add functionality for WSD
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -1002,7 +1030,7 @@ class Script(WMLResource):
            >>> script_details = client.scripts.get_revision_details(script_uid, rev_uid)
         """
         # We need to enable this once we add functionality for WSD
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -1030,7 +1058,7 @@ class Script(WMLResource):
                                                pre_defined=None,
                                                revision=rev_uid))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             entity = response[u'entity']
@@ -1078,7 +1106,7 @@ class Script(WMLResource):
                     metadata.update(
                         {'description':response_data[u'metadata'][u'description']})
 
-                if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES:
+                if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                     if "revision_id" in response_data[u'metadata']:
                         revision_id = response_data[u'metadata'][u'revision_id']
                         metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
@@ -1139,7 +1167,7 @@ class Script(WMLResource):
                                 'created_at': response_data['metadata']['created_at'],
                                 'last_updated_at': response_data['metadata']['usage']['last_updated_at']
                                 }
-                    if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES:
+                    if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                         if "revision_id" in response_data[u'metadata']:
                             revision_id = response_data[u'metadata'][u'revision_id']
                             metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
@@ -1157,7 +1185,7 @@ class Script(WMLResource):
                     new_el = {'metadata': metadata,
                               'entity': response_data['entity']
                               }
-                    if self._client.CLOUD_PLATFORM_SPACES:
+                    if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                         href_without_host = response_data['href'].split('.com')[-1]
                         new_el[u'metadata'].update({'href': href_without_host})
             return new_el
