@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2020
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -63,7 +68,8 @@ class Shiny(WMLResource):
                                       headers=self._client._get_headers(), verify=False)
         if response.status_code == 200:
             response = self._get_required_element_from_response(self._handle_response(200, u'get asset details', response))
-            if not self._client.CLOUD_PLATFORM_SPACES:
+
+            if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
                 return response
             else:
                 final_response = {
@@ -113,7 +119,7 @@ class Shiny(WMLResource):
 
         response = self._create_asset(shiny_meta, file_path)
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             final_response = {
@@ -134,7 +140,7 @@ class Shiny(WMLResource):
         else:
             desc = ""
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             asset_meta = {
                 "metadata": {
                     "name": name,
@@ -204,22 +210,31 @@ class Shiny(WMLResource):
                         json=asset_meta
                 )
             else:
-                # Until global asset for shiny is created
-                asset_type_response = requests.post(
-                    self._wml_credentials['url'] + "/v2/asset_types?",
-                    headers=self._client._get_headers(),
-                    json=base_shiny_asset,
-                    params=self._client._params(),
-                    verify=False
-                )
-                if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
+                if self._client.ICP_PLATFORM_SPACES:
                     creation_response = requests.post(
                         self._href_definitions.get_data_assets_href(),
                         headers=self._client._get_headers(),
+                        params=self._client._params(),
                         json=asset_meta,
+                        verify=False
+                    )
+                else:
+                    # Until global asset for shiny is created
+                    asset_type_response = requests.post(
+                        self._wml_credentials['url'] + "/v2/asset_types?",
+                        headers=self._client._get_headers(),
+                        json=base_shiny_asset,
                         params=self._client._params(),
                         verify=False
                     )
+                    if asset_type_response.status_code == 201 or asset_type_response.status_code == 409:
+                        creation_response = requests.post(
+                            self._href_definitions.get_data_assets_href(),
+                            headers=self._client._get_headers(),
+                            json=asset_meta,
+                            params=self._client._params(),
+                            verify=False
+                        )
 
             asset_details = self._handle_response(201, u'creating new asset', creation_response)
             #Step2: Create attachment
@@ -365,7 +380,7 @@ class Shiny(WMLResource):
                   >>> client.shiny.download(asset_uid,"shiny_asset.zip")
 
          """
-        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if rev_uid is not None and self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not applicable for this release')
 
         Shiny._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
@@ -545,7 +560,7 @@ class Shiny(WMLResource):
 
             >>> script_details = client.script.update(model_uid, meta, content_path)
         """
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         Shiny._validate_type(shiny_uid, 'shiny_uid', STR_TYPE, True)
@@ -677,7 +692,7 @@ class Shiny(WMLResource):
 
         response = self._get_required_element_from_response(self._handle_response(200, "Get shiny details", response))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             final_response = {
@@ -744,7 +759,7 @@ class Shiny(WMLResource):
 
            >>> shiny_revision = client.shiny.create_revision(shiny_uid)
         """
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -761,7 +776,7 @@ class Shiny(WMLResource):
         response = self._get_required_element_from_response(
             self._create_revision_artifact_for_assets(shiny_uid, 'Shiny'))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             final_response = {
@@ -788,7 +803,7 @@ class Shiny(WMLResource):
            >>> client.shiny.list_revisions(shiny_uid)
         """
 
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -830,7 +845,7 @@ class Shiny(WMLResource):
 
            >>> shiny_details = client.shiny.get_revision_details(shiny_uid, rev_uid)
         """
-        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 is None and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError(u'Not supported in this release')
 
         ##For CP4D, check if either space or project ID is set
@@ -858,7 +873,7 @@ class Shiny(WMLResource):
                                                pre_defined=None,
                                                revision=rev_uid))
 
-        if not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             return response
         else:
             final_response = {
@@ -939,7 +954,7 @@ class Shiny(WMLResource):
                             'created_at': response_data['metadata']['created_at'],
                             'last_updated_at': response_data['metadata']['usage']['last_updated_at']
                             }
-                if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES:
+                if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                     if "revision_id" in response_data[u'metadata']:
                         revision_id = response_data[u'metadata'][u'revision_id']
                         metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
@@ -999,7 +1014,7 @@ class Shiny(WMLResource):
                                 'last_updated_at': response_data['metadata']['usage']['last_updated_at']
                                 }
 
-                    if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES:
+                    if self._client.ICP_30 is not None or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                         if "revision_id" in response_data[u'metadata']:
                             revision_id = response_data[u'metadata'][u'revision_id']
                             metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
@@ -1017,7 +1032,7 @@ class Shiny(WMLResource):
             if 'description' in response_data['metadata']:
                 new_el[u'metadata'].update({'description': response_data[u'metadata'][u'description']})
 
-            if self._client.CLOUD_PLATFORM_SPACES and 'href' in response_data['metadata']:
+            if (self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and 'href' in response_data['metadata']:
                 href_without_host = response_data['metadata']['href'].split('.com')[-1]
                 new_el[u'metadata'].update({'href': href_without_host})
             return new_el
