@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -95,8 +100,13 @@ class SwSpec(WMLResource):
                                     params=self._client._params(skip_space_project_chk=True),
                                     headers=self._client._get_headers())
         else:
-            response = requests.get(self._href_definitions.get_sw_spec_href(sw_spec_uid), params=self._client._params(),
-                                      headers=self._client._get_headers(), verify=False)
+            if self._client.ICP_PLATFORM_SPACES:
+                response = requests.get(self._href_definitions.get_sw_spec_href(sw_spec_uid),
+                                        params=self._client._params(skip_space_project_chk=True),
+                                        headers=self._client._get_headers(), verify=False)
+            else:
+                response = requests.get(self._href_definitions.get_sw_spec_href(sw_spec_uid), params=self._client._params(),
+                                          headers=self._client._get_headers(), verify=False)
 
         if response.status_code == 200:
             return self._get_required_element_from_response(self._handle_response(200, u'get sw spec details', response))
@@ -172,7 +182,6 @@ class SwSpec(WMLResource):
         """
 
         if not self._client.WSD_20:
-            # Todo provide api to return
             href = self._href_definitions.get_sw_specs_href()
             if not self._ICP:
                 response = requests.get(href, params=self._client._params(), headers=self._client._get_headers())
@@ -437,10 +446,12 @@ class SwSpec(WMLResource):
             response = requests.put(url, params=self._client._params(), headers=self._client._get_headers())
         else:
             response = requests.put(url,  params=self._client._params(), headers=self._client._get_headers(), verify=False)
-            if response.status_code == 204:
-                return "SUCCESS"
-            else:
-                return self._handle_response(204, u'pkg spec add', response, False)
+
+        if response.status_code == 204:
+            print("SUCCESS")
+            return "SUCCESS"
+        else:
+            return self._handle_response(204, u'pkg spec add', response, False)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def delete_package_extension(self, sw_spec_uid, pkg_extn_id):
@@ -543,7 +554,7 @@ class SwSpec(WMLResource):
                           'entity': response_data['entity']
 
                           }
-            if self._client.CLOUD_PLATFORM_SPACES and 'href' in response_data['metadata']:
+            if (self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES) and 'href' in response_data['metadata']:
                 href_without_host = response_data['metadata']['href'].split('.com')[-1]
                 new_el[u'metadata'].update({'href': href_without_host})
             return new_el
