@@ -27,7 +27,6 @@ from lale.operators import TrainablePipeline
 from lale.lib.lale import Hyperopt
 
 
-@unittest.skipIf(is_cp4d(), "Not supported on CP4D")
 class TestAutoAIRemote(unittest.TestCase):
     """
     The test can be run on CLOUD, WMLS and CPD (not tested)
@@ -104,16 +103,21 @@ class TestAutoAIRemote(unittest.TestCase):
             cls.cos_endpoint = cls.cos_credentials.get('endpoint_url')
             cls.cos_resource_instance_id = cls.cos_credentials.get('resource_instance_id')
 
-        space_cleanup(cls.wml_client,
-                      get_space_id(cls.wml_client, cls.space_name,
-                                   cos_resource_instance_id=cls.cos_resource_instance_id),
-                      days_old=2)
-        cls.space_id = get_space_id(cls.wml_client, cls.space_name,
-                                    cos_resource_instance_id=cls.cos_resource_instance_id)
-        cls.project_id = None
-        cls.wml_client.set.default_space(cls.space_id)
+        cls.project_id = cls.wml_credentials.get('project_id')
 
-    def test_00_prepare_COS_instance(self):
+
+    def test_00a_space_cleanup(self):
+        space_cleanup(self.wml_client,
+                      get_space_id(self.wml_client, self.space_name,
+                                   cos_resource_instance_id=self.cos_resource_instance_id),
+                      days_old=2)
+        TestAutoAIRemote.space_id = get_space_id(self.wml_client, self.space_name,
+                                     cos_resource_instance_id=self.cos_resource_instance_id)
+        TestAutoAIRemote.project_id = None
+        TestAutoAIRemote.wml_client.set.default_space(TestAutoAIRemote.space_id)
+
+
+    def test_00b_prepare_COS_instance(self):
         if self.wml_client.ICP:
             self.skipTest("Prepare COS is available only for Cloud")
         import ibm_boto3
