@@ -27,7 +27,6 @@ from ibm_watson_machine_learning.tests.utils.cleanup import delete_model_deploym
 
 
 
-@unittest.skipIf(is_cp4d(), "Not supported on CP4D")
 class TestAutoAIRemote(unittest.TestCase):
     """
     The test covers:
@@ -111,17 +110,19 @@ class TestAutoAIRemote(unittest.TestCase):
             cls.data_location = './autoai/data/CarPrice_Assignment.csv'
             # --- end note
 
-        space_cleanup(cls.wml_client,
-                      get_space_id(cls.wml_client, cls.space_name,
-                                   cos_resource_instance_id=cls.cos_resource_instance_id),
+        cls.project_id = cls.wml_credentials.get('project_id')
+
+    def test_00a_space_cleanup(self):
+        space_cleanup(self.wml_client,
+                      get_space_id(self.wml_client, self.space_name,
+                                   cos_resource_instance_id=self.cos_resource_instance_id),
                       days_old=2)
-        cls.space_id = get_space_id(cls.wml_client, cls.space_name,
-                                    cos_resource_instance_id=cls.cos_resource_instance_id)
-        cls.project_id = None
+        TestAutoAIRemote.space_id = get_space_id(self.wml_client, self.space_name,
+                                     cos_resource_instance_id=self.cos_resource_instance_id)
+        TestAutoAIRemote.project_id = None
+        TestAutoAIRemote.wml_client.set.default_space(TestAutoAIRemote.space_id)
 
-        cls.wml_client.set.default_space(cls.space_id)
-
-    def test_00_prepare_COS_instance(self):
+    def test_00b_prepare_COS_instance(self):
         if self.wml_client.ICP or self.wml_client.WSD:
             self.skipTest("Prepare COS is available only for Cloud")
         import ibm_boto3
