@@ -21,7 +21,6 @@ from ibm_watson_machine_learning.tests.utils import get_wml_credentials, get_cos
 from ibm_watson_machine_learning.tests.utils.cleanup import delete_model_deployment, bucket_cleanup, space_cleanup
 
 
-@unittest.skipIf(is_cp4d(), "Not supported on CP4D")
 class TestAutoAIRemote(unittest.TestCase):
     """
     The test covers:
@@ -105,17 +104,19 @@ class TestAutoAIRemote(unittest.TestCase):
             cls.data_location = './autoai/data/iris_dataset.csv'
             # -- end note
 
+        cls.project_id = cls.wml_credentials.get('project_id')
 
-        space_cleanup(cls.wml_client,
-                      get_space_id(cls.wml_client, cls.space_name, cos_resource_instance_id=cls.cos_resource_instance_id),
+    def test_00a_space_cleanup(self):
+        space_cleanup(self.wml_client,
+                      get_space_id(self.wml_client, self.space_name,
+                                   cos_resource_instance_id=self.cos_resource_instance_id),
                       days_old=2)
-        cls.space_id = get_space_id(cls.wml_client, cls.space_name, cos_resource_instance_id=cls.cos_resource_instance_id)
-        cls.project_id = None
+        TestAutoAIRemote.space_id = get_space_id(self.wml_client, self.space_name,
+                                                 cos_resource_instance_id=self.cos_resource_instance_id)
+        TestAutoAIRemote.project_id = None
+        TestAutoAIRemote.wml_client.set.default_space(TestAutoAIRemote.space_id)
 
-        cls.wml_client = APIClient(wml_credentials=cls.wml_credentials.copy())
-        cls.wml_client.set.default_space(cls.space_id)
-
-    def test_00_prepare_COS_instance(self):
+    def test_00b_prepare_COS_instance(self):
         if self.wml_client.ICP or self.wml_client.WSD:
             self.skipTest("Prepare COS is available only for Cloud")
         import ibm_boto3
@@ -466,8 +467,14 @@ class TestAutoAIRemote(unittest.TestCase):
     #########################
 
     def test_30_batch_deployment_setup_and_preparation(self):
-        TestAutoAIRemote.service_batch = Batch(self.wml_credentials,
-                                               source_space_id=self.space_id)
+        if self.wml_client.ICP:
+            TestAutoAIRemote.service_batch = Batch(source_wml_credentials=self.wml_credentials.copy(),
+                                                   source_project_id=self.project_id,
+                                                   target_wml_credentials=self.wml_credentials,
+                                                   target_space_id=self.space_id)
+        else:
+            TestAutoAIRemote.service_batch = Batch(self.wml_credentials,
+                                                   source_space_id=self.space_id)
 
         self.assertIsInstance(self.service_batch, Batch, msg="Deployment is not of Batch type.")
         self.assertIsInstance(self.service_batch._source_workspace, WorkSpace, msg="Workspace set incorrectly.")
