@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import print_function
 import requests
@@ -29,7 +34,7 @@ class Training(WMLResource):
         WMLResource.__init__(self, __name__, client)
         self._ICP = client.ICP
         self.ConfigurationMetaNames = TrainingConfigurationMetaNames()
-        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             self.ConfigurationMetaNames = TrainingConfigurationMetaNamesCp4d30()
 
     @staticmethod
@@ -85,7 +90,7 @@ class Training(WMLResource):
 
            **Parameters**
 
-           .. important::
+            .. important::
                 #. **training_uid**: Unique Id of Training (optional)\n
                    **type**: str\n
                 #. **limit**:  limit number of fetched records (optional)\n
@@ -93,7 +98,7 @@ class Training(WMLResource):
 
            **Output**
 
-           .. important::
+            .. important::
                 **returns**: metadata of training(s)\n
                 **return type**: dict
                 The output can be {"resources": [dict]} or a dict\n
@@ -208,14 +213,14 @@ class Training(WMLResource):
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def run(self, meta_props, asynchronous=True):
         """
-                Create a new Machine Learning training.
+            Create a new Machine Learning training.
 
-                **Parameters**
+            **Parameters**
 
                 .. important::
 
                     #. **meta_props**:  meta data of the training configuration. To see available meta names use:\n
-                                        >>> client.training.ConfigurationMetaNames.show()
+                            >>> client.training.ConfigurationMetaNames.show()
 
                        **type**: str\n
 
@@ -225,14 +230,14 @@ class Training(WMLResource):
                        **type**: bool\n
 
 
-                **Output**
+            **Output**
 
                 .. important::
 
                     **returns**: Metadata of the training created\n
                     **return type**: dict\n
 
-                **Examples**
+            **Examples**
                     **Example meta_props for Training run creation in Cloud Pack for Data version 3.1.0 or above:
                     >>> metadata = {
                     >>>  client.training.ConfigurationMetaNames.NAME: 'Hand-written Digit Recognition',
@@ -317,7 +322,9 @@ class Training(WMLResource):
                     >>> }
                     >>> training_details = client.training.run(definition_uid, meta_props=metadata)
                     >>> training_uid = client.training.get_uid(training_details)
+
         """
+
         ##For CP4D, check if either spce or project ID is set
         self._client._check_if_either_is_set()
         Training._validate_type(meta_props, 'meta_props', object, True)
@@ -334,7 +341,7 @@ class Training(WMLResource):
 
         # TODO remove when training service starts copying such data on their own
 
-        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.ICP_30 or self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             training_configuration_metadata["name"] = meta_props[self.ConfigurationMetaNames.NAME]
             training_configuration_metadata["description"] = meta_props[self.ConfigurationMetaNames.DESCRIPTION]
 
@@ -433,7 +440,19 @@ class Training(WMLResource):
                                                 headers=self._client._get_headers())
 
         else:
-            if self._client.ICP_30:
+            if self._client.ICP_PLATFORM_SPACES:
+                params = self._client._params()
+                if 'space_id' in params.keys():
+                    params.pop('space_id')
+                if 'project_id' in params.keys():
+                    params.pop('project_id')
+                response_train_post = requests.post(train_endpoint,
+                                                    json=training_configuration_metadata,
+                                                    params= params,
+                                                    headers=self._client._get_headers(),
+                                                    verify=False)
+
+            elif self._client.ICP_30:
                 response_train_post = requests.post(train_endpoint, json=training_configuration_metadata,
                                                     headers=self._client._get_headers(),verify=False)
             else:
@@ -516,7 +535,7 @@ class Training(WMLResource):
 
         """
         ##For CP4D, check if either spce or project ID is set
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Sub-trainings are no longer available for trainings, please use list_intermediate_models().")
 
         self._client._check_if_either_is_set()
@@ -565,7 +584,7 @@ class Training(WMLResource):
 
         """
         ##For CP4D, check if either spce or project ID is set
-        if self._client.ICP_30:
+        if self._client.ICP_30 or self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("This method is not supported for IBM Cloud Pack for Data. " )
 
         self._client._check_if_either_is_set()
@@ -647,19 +666,19 @@ class Training(WMLResource):
         self._client._check_if_either_is_set()
         training_uid = str_type_conv(training_uid)
         Training._validate_type(training_uid, u'training_uid', STR_TYPE, True)
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             params = self._client._params()
         else:
             params = None
 
         if hard_delete is True:
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 params.update({'hard_delete': u'true'})
             else:
                 params = {}
                 params.update({'hard_delete': u'true'})
 
-        if not self._ICP and not self._client.CLOUD_PLATFORM_SPACES:
+        if not self._ICP and not self._client.CLOUD_PLATFORM_SPACES and not self._client.ICP_PLATFORM_SPACES:
             response_delete = requests.delete(self._href_definitions.get_training_href(training_uid),
                                           headers=self._client._get_headers(),params=params)
         else:
@@ -820,7 +839,7 @@ class Training(WMLResource):
 
         """
 
-        if self._client.ICP_30:
+        if self._client.ICP_30 or self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Metrics logs are not supported. This method is not supported for IBM Cloud Pack for Data. ")
 
         training_uid = str_type_conv(training_uid)
@@ -914,7 +933,7 @@ class Training(WMLResource):
 
              >>> client.training.monitor_metrics(training_uid)
         """
-        if self._client.ICP_30:
+        if self._client.ICP_30 or self._client.ICP_PLATFORM_SPACES:
             raise WMLClientError("Metrics monitoring is not supported for IBM Cloud Pack for Data")
 
         training_uid = str_type_conv(training_uid)
@@ -1001,7 +1020,7 @@ class Training(WMLResource):
 
         training_uid = str_type_conv(training_uid)
         Training._validate_type(training_uid, u'training_uid', STR_TYPE, True)
-        if self._client.CLOUD_PLATFORM_SPACES:
+        if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
             entity = self.get_details(training_uid)
             if 'metrics' in entity:
                 metrics = entity['metrics']
