@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 __all__ = [
     "MissingPipeline",
     "FitNotCompleted",
@@ -29,7 +43,9 @@ __all__ = [
     'TrainingDataSourceIsNotFile',
     'VisualizationFailed',
     'InvalidPredictionType',
-    'DataLocationTypeNotSupported'
+    'DataLocationTypeNotSupported',
+    'InvalidIdType',
+    'NoneDataConnection'
 ]
 
 from ibm_watson_machine_learning.utils import WMLClientError
@@ -197,4 +213,14 @@ class InvalidPredictionType(WMLClientError, ValueError):
 
 class DataLocationTypeNotSupported(WMLClientError, ValueError):
     def __init__(self, value_name=None, reason=None):
-        WMLClientError.__init__(self, f"Data Location type: {value_name} is not supported for this action", reason)
\ No newline at end of file
+        WMLClientError.__init__(self, f"Data Location type: {value_name} is not supported for this action", reason)
+
+
+class InvalidIdType(WMLClientError, ValueError):
+    def __init__(self, typ):
+        WMLClientError.__init__(self, f"Unexpected type of id: {typ}")
+
+
+class NoneDataConnection(WMLClientError, ValueError):
+    def __init__(self, resource_name):
+        WMLClientError.__init__(self, f"Invalid DataConnection in {resource_name} list. DataConnection cannot be None")
\ No newline at end of file
