@@ -1,37 +1,23 @@
-import io
-import json
-import os
-from zipfile import ZipFile
-from contextlib import redirect_stdout
-from subprocess import check_call
-from sys import executable
-from tarfile import open as open_tar
-from time import sleep
-from typing import Dict, Union, Tuple, List, TYPE_CHECKING
-from warnings import warn
-from functools import wraps
-
-import requests
-from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
-
-from .enums import RunStateTypes
-from .errors import (MissingPipeline, AutoAIComputeError, DataFormatNotSupported, LibraryNotCompatible,
-                     CannotInstallLibrary, CannotDownloadTrainingDetails, CannotDownloadWMLPipelineDetails,
-                     VisualizationFailed)
-
-if TYPE_CHECKING:
-    from io import BytesIO, BufferedIOBase
-    from pandas import DataFrame
-    from collections import OrderedDict
-    from sklearn.pipeline import Pipeline
-    from ibm_watson_machine_learning import APIClient
-    from ibm_watson_machine_learning.helpers import DataConnection, S3Connection
-    from ibm_boto3 import resource, client
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 __all__ = [
     'fetch_pipelines',
     'load_file_from_file_system',
     'NextRunDetailsGenerator',
+    'prepare_auto_ai_model_to_publish_normal_scenario',
+    'prepare_auto_ai_model_to_publish_notebook_normal_scenario',
     'prepare_auto_ai_model_to_publish',
     'remove_file',
     'RunDetailsGenerator',
@@ -52,9 +38,41 @@ __all__ = [
     'prepare_model_location_path',
     'download_wml_pipeline_details_from_file',
     'init_cos_client',
-    'check_graphviz_binaries'
+    'check_graphviz_binaries',
+    'try_import_joblib',
+    'get_sw_spec_and_type_based_on_sklearn'
 ]
 
+import io
+import json
+import os
+from zipfile import ZipFile
+from contextlib import redirect_stdout
+from subprocess import check_call
+from sys import executable
+from tarfile import open as open_tar
+from time import sleep
+from typing import Dict, Union, Tuple, List, TYPE_CHECKING, Optional
+from warnings import warn
+from functools import wraps
+
+import requests
+from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
+
+from .enums import RunStateTypes
+from .errors import (MissingPipeline, AutoAIComputeError, DataFormatNotSupported, LibraryNotCompatible,
+                     CannotInstallLibrary, CannotDownloadTrainingDetails, CannotDownloadWMLPipelineDetails,
+                     VisualizationFailed)
+
+if TYPE_CHECKING:
+    from io import BytesIO, BufferedIOBase
+    from pandas import DataFrame
+    from collections import OrderedDict
+    from sklearn.pipeline import Pipeline
+    from ibm_watson_machine_learning import APIClient
+    from ibm_watson_machine_learning.helpers import DataConnection, S3Connection
+    from ibm_boto3 import resource, client
+
 
 def create_model_download_link(file_path: str):
     """
@@ -116,7 +134,7 @@ def fetch_pipelines(run_params: dict,
                 check_dependencies_versions(lib_name='lightgbm', error=True)
         # --- end note
 
-    from sklearn.externals import joblib
+    joblib = try_import_joblib()
 
     path = os.path.abspath(path)
     pipelines_names = []
@@ -318,9 +336,49 @@ class NextRunDetailsGenerator:
             raise StopIteration
 
 
-def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline', 'TrainablePipeline'],
-                                              result_connection,
-                                              cos_client) -> Union[Tuple[Dict[str, dict], str], str]:
+def preprocess_request_json(request_json: Dict, space_id: str) -> Dict:
+    """Removes unused parts of request.json file got from autoai training.
+    Allow to further store model in user space."""
+    # note: if training was on project_id, change it to space_id as we can deploy only on space
+    if 'project_id' in request_json:
+        request_json.pop('project_id')
+
+    request_json['space_id'] = space_id
+    request_json.pop('pipeline')  # not needed for other space
+    request_json.pop('training_data_references')  # not needed for other space
+    # --- end note
+    return request_json
+
+
+def chose_model_output(model_number: str) -> str:
+    """Chose correct path for particular model number"""
+    model_number = int(model_number)
+    hpo_c_numbers = (4, 8, 12, 16)
+    cognito_numbers = (3, 7, 11, 15)
+    hpo_d_numbers = (2, 6, 10, 14)
+    pre_hpo_d_numbers = (1, 5, 9, 13)
+
+    if model_number in pre_hpo_d_numbers:
+        return 'pre_hpo_d_output'
+
+    elif model_number in hpo_d_numbers:
+        return 'hpo_d_output'
+
+    elif model_number in cognito_numbers:
+        return 'cognito_output'
+
+    elif model_number in hpo_c_numbers:
+        return 'hpo_c_output'
+
+    else:
+        return 'global_output'
+
+
+def prepare_auto_ai_model_to_publish_notebook_normal_scenario(
+        pipeline_model: Union['Pipeline', 'TrainablePipeline'],
+        result_connection,
+        cos_client,
+        space_id: str) -> Union[Tuple[str, Dict[str, dict]]]:
     """
     Prepares autoai model to publish in Watson Studio via COS.
     Option only for auto-gen notebooks with correct result references on COS.
@@ -336,17 +394,66 @@ def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline',
     cos_client: ibm_boto3.resource, required
         initialized COS client
 
+    space_id: str, required
+
     Returns
     -------
     String with path to the saved model and jsons in COS.
     """
-    from sklearn.externals import joblib
+    path = result_connection.location._model_location
+    model_number = pipeline_model.split('_')[-1]
+    run_id = path.split('/data/')[0].split('/')[-1]
+    request_path = f"{path.split('/data/')[0]}/assets/{run_id}_P{model_number}_{chose_model_output(model_number)}/resources/wml_model/request.json"
+
+    bucket = result_connection.location.bucket
+    cos_client.meta.client.download_file(Bucket=bucket, Filename='request.json', Key=request_path)
+
+    with open('request.json', 'r') as f:
+        request_str = f.read()
+
+    request_json: Dict[str, dict] = json.loads(request_str)
+    request_json['content_location']['connection'] = result_connection.connection.to_dict()
+    request_json = preprocess_request_json(request_json, space_id)
+    artifact_name = f"autoai_sdk{os.path.sep}{pipeline_model}.pickle"
+
+    return artifact_name, request_json
+
+
+# TODO: remove this function
+def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline', 'TrainablePipeline'],
+                                              result_connection,
+                                              cos_client,
+                                              obm: Optional[bool] = False) -> Union[Tuple[Dict[str, dict], str], str]:
+    """
+    Prepares autoai model to publish in Watson Studio via COS.
+    Option only for auto-gen notebooks with correct result references on COS.
+
+    Parameters
+    ----------
+    pipeline_model: Union['Pipeline', 'TrainablePipeline'], required
+        model object to publish
+
+    result_connection: DataConnection, required
+        Connection object with COS credentials and all needed locations for jsons
+
+    cos_client: ibm_boto3.resource, required
+        initialized COS client
+
+    obm: bool, optional
+        Indicator if we need to extract OBM data
+
+    Returns
+    -------
+    String with path to the saved model and jsons in COS.
+    """
+    joblib = try_import_joblib()
 
     artifact_type = ".gzip"
 
     artifact_name = f"artifact_auto_ai_model{artifact_type}"
     model_artifact_name = f"model_.tar.gz"
     wml_pipeline_definition_name = "pipeline-model.json"
+    obm_model_name = "obm_model.zip"
     temp_model_name = '__temp_model.pickle'
 
     # note: path to the json describing the autoai POD specification
@@ -356,6 +463,12 @@ def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline',
 
     bucket = result_connection.location.bucket
 
+    # note: Check if we have OBM experiment and get paths for obm model and schema
+    if obm:
+        obm_model_path = f"{path.split('/data/')[0]}/data/obm/model.zip"
+        schema_path = f"{path.split('/data/')[0]}/data/obm/schemas.json"
+        cos_client.meta.client.download_file(Bucket=bucket, Filename=obm_model_name, Key=obm_model_path)
+
     # note: need to download model schema and wml pipeline definition json
     cos_client.meta.client.download_file(Bucket=bucket, Filename=wml_pipeline_definition_name,
                                          Key=pipeline_model_json_path)
@@ -372,7 +485,6 @@ def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline',
     joblib.dump(pipeline_model, temp_model_name)
     # --- end note
 
-
     # note: create a tar.gz file with model pickle, name it as 'model_run_id.tar.gz', model.pickle inside
     with open_tar(model_artifact_name, 'w:gz') as tar:
         tar.add(temp_model_name, arcname='model.pickle')
@@ -382,115 +494,185 @@ def prepare_auto_ai_model_to_publish_notebook(pipeline_model: Union['Pipeline',
 
     # note: create final zip to publish on WML cloud v4 GA
     with ZipFile(artifact_name, 'w') as zip_file:
+        if obm:
+            # note: write order is important!
+            zip_file.write(obm_model_name)
         zip_file.write(model_artifact_name)
         zip_file.write(wml_pipeline_definition_name)
 
     remove_file(filename=model_artifact_name)
     remove_file(filename=wml_pipeline_definition_name)
+    if obm:
+        remove_file(filename=obm_model_name)
     # --- end note
 
     return json.loads(schema_json), artifact_name
 
 
+def prepare_auto_ai_model_to_publish_normal_scenario(
+        pipeline_model: Union['Pipeline', 'TrainablePipeline'],
+        run_params: dict,
+        run_id: str,
+        wml_client: 'APIClient',
+        space_id: str) -> Union[Tuple[str, Dict[str, dict]]]:
+    """
+    Helper function to specify `content_location` statement for AutoAI models to store in repository.
+
+    Parameters
+    ----------
+    pipeline_model: Union['Pipeline', 'TrainablePipeline'], required
+        Model that will be prepared for an upload.
+
+    run_params: dict, required
+        Fetched details of the run/fit.
+
+    run_id: str, required
+        Fit/run ID associated with the model.
+
+    wml_client: APIClient, required
+
+    space_id: str, required
+
+    Returns
+    -------
+    If cp4d: Dictionary with model schema and artifact name to upload, stored temporally in the user local file system.
+    else: path name to the stored model in COS
+    """
+
+    pipeline_info = run_params['entity']['status'].get('metrics')[-1]
+    schema_path = f"{pipeline_info['context']['intermediate_model']['schema_location']}"
+    model_number = pipeline_model.split('_')[-1]
+    request_path = f"{schema_path.split('/data/')[0]}/assets/{run_id}_P{model_number}_{chose_model_output(model_number)}/resources/wml_model/request.json"
+
+    if wml_client.ICP:
+        request_str = load_file_from_file_system(wml_client=wml_client, file_path=request_path).read().decode()
+
+    else:
+        cos_client = init_cos_client(run_params['entity']['results_reference']['connection'])
+        bucket = run_params['entity']['results_reference']['location']['bucket']
+        cos_client.meta.client.download_file(Bucket=bucket, Filename='request.json', Key=request_path)
+        with open('request.json', 'r') as f:
+            request_str = f.read()
+
+    request_json: Dict[str, dict] = json.loads(request_str)
+    # note: fill connection details
+    request_json['content_location']['connection'] = run_params['entity']['results_reference']['connection']
+    # note: if training was on project_id, change it to space_id as we can deploy only on space
+    request_json = preprocess_request_json(request_json, space_id)
+    artifact_name = f"autoai_sdk{os.path.sep}{pipeline_model}.pickle"
+
+    return artifact_name, request_json
+
+
+# TODO: remove this function
 def prepare_auto_ai_model_to_publish(
         pipeline_model: Union['Pipeline', 'TrainablePipeline'],
         run_params: dict,
         run_id: str,
-
         wml_client: 'APIClient') -> Union[Tuple[Dict[str, dict], str], str]:
-        """
-        Helper function to download and load computed AutoAI pipelines (sklearn pipelines).
+    """
+    Helper function to download and load computed AutoAI pipelines (sklearn pipelines).
+    Parameters
+    ----------
+    pipeline_model: Union['Pipeline', 'TrainablePipeline'], required
+        Model that will be prepared for an upload.
+    run_params: dict, required
+        Fetched details of the run/fit.
+    run_id: str, required
+        Fit/run ID associated with the model.
+    wml_client: APIClient, required
+    Returns
+    -------
+    If cp4d: Dictionary with model schema and artifact name to upload, stored temporally in the user local file system.
+    else: path name to the stored model in COS
+    """
 
-        Parameters
-        ----------
-        pipeline_model: Union['Pipeline', 'TrainablePipeline'], required
-            Model that will be prepared for an upload.
+    joblib = try_import_joblib()
 
-        run_params: dict, required
-            Fetched details of the run/fit.
+    artifact_type = ".tar.gz" if wml_client.ICP else ".gzip"
 
-        run_id: str, required
-            Fit/run ID associated with the model.
+    artifact_name = f"artifact_auto_ai_model{artifact_type}"
+    model_artifact_name = f"model_{run_id}.tar.gz"
+    wml_pipeline_definition_name = "pipeline-model.json"
+    obm_model_name = "obm_model.zip"
+    temp_model_name = '__temp_model.pickle'
 
-        wml_client: APIClient, required
+    # note: prepare file paths of pipeline-model and schema (COS / file system location)
+    pipeline_info = run_params['entity']['status'].get('metrics')[-1]
+    pipeline_model_path = f"{pipeline_info['context']['intermediate_model']['location']['pipeline_model']}"
+    schema_path = f"{pipeline_info['context']['intermediate_model']['schema_location']}"
+    obm_model_path = None
+    # --- end note
 
-        Returns
-        -------
-        If cp4d: Dictionary with model schema and artifact name to upload, stored temporally in the user local file system.
-        else: path name to the stored model in COS
-        """
+    # note: Check if we have OBM experiment and get paths for obm model and schema
+    if 'obm' in run_params['entity']['status'].get('feature_engineering_components', {}):
+        obm_model_path = f"{pipeline_model_path.split('/data/')[0]}/data/obm/model.zip"
+        schema_path = f"{pipeline_model_path.split('/data/')[0]}/data/obm/schemas.json"
 
-        from sklearn.externals import joblib
-
-        artifact_type = ".tar.gz" if wml_client.ICP else ".gzip"
-
-        artifact_name = f"artifact_auto_ai_model{artifact_type}"
-        model_artifact_name = f"model_{run_id}.tar.gz"
-        wml_pipeline_definition_name = "pipeline-model.json"
-        temp_model_name = '__temp_model.pickle'
-
-        # note: prepare file paths of pipeline-model and schema (COS / file system location)
-        pipeline_info = run_params['entity']['status'].get('metrics')[-1]
-        pipeline_model_path = f"{pipeline_info['context']['intermediate_model']['location']['pipeline_model']}"
-        schema_path = f"{pipeline_info['context']['intermediate_model']['schema_location']}"
+    if wml_client.ICP:
+        # note: downloading pipeline-model.json and schema.json from file system on CP4D
+        schema_json = load_file_from_file_system(wml_client=wml_client, file_path=schema_path).read().decode()
+        pipeline_model_json = load_file_from_file_system(wml_client=wml_client,
+                                                         file_path=pipeline_model_path).read().decode()
+        with open(wml_pipeline_definition_name, 'w') as f:
+            f.write(pipeline_model_json)
         # --- end note
 
-        if wml_client.ICP:
-            # note: downloading pipeline-model.json and schema.json from file system on CP4D
-            schema_json = load_file_from_file_system(wml_client=wml_client, file_path=schema_path).read().decode()
-            pipeline_model_json = load_file_from_file_system(wml_client=wml_client,
-                                                             file_path=pipeline_model_path).read().decode()
-            with open(wml_pipeline_definition_name, 'w') as f:
-                f.write(pipeline_model_json)
-            # --- end note
+        # note: save obm model.zip locally
+        if obm_model_path is not None:
+            obm_model = load_file_from_file_system(wml_client=wml_client,
+                                                   file_path=obm_model_path).read().decode()
 
-            # note: update the schema, it has wrong field types
-            schema_json = schema_json.replace('fieldType', 'type')
-            # --- end note
-
-        else:
-            cos_client = init_cos_client(run_params['entity']['results_reference']['connection'])
-            bucket = run_params['entity']['results_reference']['location']['bucket']
-
-            # note: need to download model schema and wml pipeline definition json
-            cos_client.meta.client.download_file(Bucket=bucket, Filename=wml_pipeline_definition_name,
-                                                 Key=pipeline_model_path)
-            cos_client.meta.client.download_file(Bucket=bucket, Filename='schema.json', Key=schema_path)
-
-            with open('schema.json', 'r') as f:
-                schema_json = f.read()
-
-            # note: update the schema, it has wrong field types
-            schema_json = schema_json.replace('fieldType', 'type')
-            # --- end note
-
-        # note: saved passed model as pickle, for further tar.gz packaging
-        joblib.dump(pipeline_model, temp_model_name)
+            with open(obm_model_name, 'w') as f:
+                f.write(obm_model)
         # --- end note
 
-        # note: create a tar.gz file with model pickle, name it as 'model_run_id.tar.gz', model.pickle inside
-        with open_tar(model_artifact_name, 'w:gz') as tar:
-            tar.add(temp_model_name, arcname='model.pickle')
+    else:
+        cos_client = init_cos_client(run_params['entity']['results_reference']['connection'])
+        bucket = run_params['entity']['results_reference']['location']['bucket']
 
-        remove_file(filename=temp_model_name)
+        # note: need to download model schema and wml pipeline definition json
+        cos_client.meta.client.download_file(Bucket=bucket, Filename=wml_pipeline_definition_name,
+                                             Key=pipeline_model_path)
+        cos_client.meta.client.download_file(Bucket=bucket, Filename='schema.json', Key=schema_path)
+
+        with open('schema.json', 'r') as f:
+            schema_json = f.read()
+
+        # note: save obm model.zip locally
+        if obm_model_path is not None:
+            cos_client.meta.client.download_file(Bucket=bucket, Filename=obm_model_name, Key=obm_model_path)
         # --- end note
 
-        if wml_client.ICP:
-            # note: create final tar.gz to publish on WML (includes pipeline config and tar.gz with model)
-            with open_tar(artifact_name, 'w:gz') as tar:
-                tar.add(model_artifact_name)
-                tar.add(wml_pipeline_definition_name)
-        else:
-            # note: create final zip to publish on WML cloud v4 GA
-            with ZipFile(artifact_name, 'w') as zip_file:
-                zip_file.write(model_artifact_name)
-                zip_file.write(wml_pipeline_definition_name)
+    # note: update the schema, it has wrong field types and missing id
+    schema_json = schema_json.replace('fieldType', 'type')
+    # --- end note
 
-        remove_file(filename=model_artifact_name)
-        remove_file(filename=wml_pipeline_definition_name)
-        # --- end note
+    # note: saved passed model as pickle, for further tar.gz packaging
+    joblib.dump(pipeline_model, temp_model_name)
+    # --- end note
 
-        return json.loads(schema_json), artifact_name
+    # note: create a tar.gz file with model pickle, name it as 'model_run_id.tar.gz', model.pickle inside
+    with open_tar(model_artifact_name, 'w:gz') as tar:
+        tar.add(temp_model_name, arcname='model.pickle')
+
+    remove_file(filename=temp_model_name)
+    # --- end note
+
+    with ZipFile(artifact_name, 'w') as zip_file:
+        if obm_model_path is not None:
+            # note: write order is important!
+            zip_file.write(obm_model_name)
+        zip_file.write(model_artifact_name)
+        zip_file.write(wml_pipeline_definition_name)
+
+    remove_file(filename=model_artifact_name)
+    remove_file(filename=wml_pipeline_definition_name)
+    if obm_model_path is not None:
+        remove_file(filename=obm_model_name)
+    # --- end note
+
+    return json.loads(schema_json), artifact_name
 
 
 def modify_pipeline_model_json(data_location: str, model_path: str) -> None:
@@ -634,7 +816,7 @@ def try_import_lale():
     """
     Check if lale package is installed in local environment, if not, just download and install it.
     """
-    lale_version = '0.3.18'
+    lale_version = '0.4.1'
     try:
         import lale
         from packaging import version
@@ -748,8 +930,54 @@ def try_import_graphviz():
                                        reason="graphviz==0.14 failed to install. Please install it manually.")
 
 
-def try_load_dataset(buffer: Union['BytesIO', 'BufferedIOBase'], sheet_name: str = 0, separator: str = ',') -> Union[
-    'DataFrame', 'OrderedDict']:
+def try_import_joblib():
+    """
+    Check if joblib is available from scikit-learn or externally and change 'load' method to inform the user about
+    compatibility issues.
+    """
+
+    def fallback_load(f):
+        @wraps(f)
+        def _f(self, *method_args, **method_kwargs):
+            try:
+                output = f(*method_args, **method_kwargs)
+
+            except:
+                import sklearn
+                if '0.20' in str(sklearn.__version__):
+                    raise LibraryNotCompatible(
+                        reason=f"Installed scikit-learn version: {sklearn.__version__} is not compatible with "
+                               f"version included in AutoAI pipeline. Please try to install another 0.23.2 version.")
+                else:
+                    raise LibraryNotCompatible(
+                        reason=f"Installed scikit-learn version: {sklearn.__version__} is not compatible with "
+                               f"version included in AutoAI pipeline. Please try to install another 0.20.3 version.")
+
+            return output
+
+        return _f
+
+    from types import MethodType
+
+    try:
+        # note only up to scikit version 0.20.3
+        from sklearn.externals import joblib
+
+    except ImportError:
+        # only for scikit 0.23.*
+        import joblib
+
+    new_load = fallback_load(joblib.load)
+    joblib.load = MethodType(new_load, joblib)
+
+    return joblib
+
+
+def try_load_dataset(
+        buffer: Union['BytesIO', 'BufferedIOBase'],
+        sheet_name: str = 0,
+        separator: str = ',',
+        encoding: Optional[str] = 'utf-8') -> Union['DataFrame', 'OrderedDict']:
     """
     Load data into a pandas DataFrame from BytesIO object.
 
@@ -764,6 +992,8 @@ def try_load_dataset(buffer: Union['BytesIO', 'BufferedIOBase'], sheet_name: str
     separator: str, optional
         csv separator
 
+    encoding: str, optional
+
     Returns
     -------
     DataFrame or OrderedDict
@@ -772,7 +1002,7 @@ def try_load_dataset(buffer: Union['BytesIO', 'BufferedIOBase'], sheet_name: str
 
     try:
         buffer.seek(0)
-        data = read_csv(buffer, sep=separator, encoding='utf-8')
+        data = read_csv(buffer, sep=separator, encoding=encoding)
 
     except Exception as e1:
         try:
@@ -801,8 +1031,8 @@ def check_dependencies_versions(lib_name: str = None, error: bool = False, msg:
         Error message.
     """
 
-    scikit_version = '0.20.3'
-    numpy_version = '1.18.2'
+    scikit_version = '0.23.2'
+    numpy_version = '1.19.1'
     xgboost_version = '0.90'
     lightgbm_version = '2.2.3'
 
@@ -1103,3 +1333,25 @@ def check_graphviz_binaries(f):
         return output
 
     return _f
+
+
+def get_sw_spec_and_type_based_on_sklearn(client: 'APIClient') -> Tuple[str, str]:
+    """Based on user environment, check sklearn version and find apropriate sw spec.
+
+    Returns
+    -------
+    model_type, sw_spec
+    """
+    import sklearn
+    if '0.20.' in sklearn.__version__:
+        sw_spec = client.software_specifications.get_id_by_name('autoai-kb_3.0-py3.6')
+        model_type = 'scikit-learn_0.20'
+
+    elif '0.23.' in sklearn.__version__:
+        sw_spec = client.software_specifications.get_id_by_name('autoai-kb_3.1-py3.6')
+        model_type = 'scikit-learn_0.23'
+
+    else:
+        raise LibraryNotCompatible(reason="Your version of scikit-learn is not supported. Use one of [0.20.*, 0.23.*]")
+
+    return model_type, sw_spec
