@@ -1,11 +1,16 @@
-################################################################################
+# (C) Copyright IBM Corp. 2020.
 #
-# Licensed Materials - Property of IBM
-# (C) Copyright IBM Corp. 2017
-# US Government Users Restricted Rights - Use, duplication disclosure restricted
-# by GSA ADP Schedule Contract with IBM Corp.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-################################################################################
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 import requests
 from ibm_watson_machine_learning.wml_client_error import MissingValue, WMLClientError, NoWMLCredentialsProvided, ApiRequestFailure, UnexpectedType, MissingMetaProp
@@ -32,7 +37,8 @@ class WMLResource:
         self._WSD = client.WSD
 
         # ToDO why this is differnt , check in 2.5 cluster and cloud whether it is working
-        if client.ICP or client.WSD:
+
+        if (client.ICP or client.WSD) and not client.ICP_PLATFORM_SPACES:
             self._wml_credentials = client.wml_credentials
             self._href_definitions = HrefDefinitions(client.wml_credentials)
         else:
@@ -40,11 +46,12 @@ class WMLResource:
             self._href_definitions = HrefDefinitions(client.service_instance._wml_credentials,
                                                      self._client.CLOUD_PLATFORM_SPACES,
                                                      self._client.PLATFORM_URL,
-                                                     self._client.CAMS_URL)
+                                                     self._client.CAMS_URL,
+                                                     self._client.ICP_PLATFORM_SPACES)
 
     def _handle_response(self, expected_status_code, operationName, response, json_response=True):
         if "dele" in operationName or "cancel migration" == operationName:
-            if self._client.CLOUD_PLATFORM_SPACES:
+            if self._client.CLOUD_PLATFORM_SPACES or self._client.ICP_PLATFORM_SPACES:
                 if response.status_code == expected_status_code:
                     return "SUCCESS"
                 else:
@@ -640,8 +647,14 @@ class WMLResource:
                 response = requests.post(url, headers=self._client._get_headers(),
                                          json=input_json)
         else:
-            response = requests.post(url, headers=self._client._get_headers(),
-                                     json=input_json, verify=False)
+            if self._client.ICP_PLATFORM_SPACES:
+                response = requests.post(url, headers=self._client._get_headers(),
+                                         params=self._client._params(),
+                                         json=input_json, verify=False)
+            else:
+                response = requests.post(url, headers=self._client._get_headers(),
+                                         json=input_json, verify=False)
+
         return self._handle_response(201, op_name, response)
 
     def _create_revision_artifact_for_assets(self, uid, resource_name):
