@@ -1,3 +1,17 @@
+# (C) Copyright IBM Corp. 2020.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from warnings import warn
 
 from ibm_watson_machine_learning import APIClient
@@ -81,8 +95,7 @@ class WorkSpace:
                 self.WMLS = True
 
         elif wml_credentials.get('instance_id', '').lower() in ('icp', 'openshift'):
-            print("Connecting to CP4D...")
-            if self.project_id is None:
+            if self.project_id is None and self.space_id is None:
                 raise MissingArgument(
                     'project_id',
                     reason="These credentials are from CP4D environment, "
@@ -90,11 +103,13 @@ class WorkSpace:
             else:
                 outcome = self.wml_client.set.default_project(self.project_id)
                 if outcome == 'FAILURE':
-                    raise SetIDFailed(
-                        'space_id',
-                        reason=f"This project_id: {self.project_id} cannot be found in current environment.")
+                    outcome = self.wml_client.set.default_space(self.space_id)
+                    if outcome == 'FAILURE':
+                        raise SetIDFailed(
+                            'project_id',
+                            reason=f"This project_id: {self.project_id} cannot be found in current environment.")
 
-            supported_versions = ('3.0.0', '3.0.1')
+            supported_versions = ('3.0.0', '3.0.1', '3.5')
             if wml_credentials[u'version'] not in supported_versions:
                 raise WrongEnvironmentVersion(wml_credentials[u'version'], wml_credentials[u'instance_id'].lower(),
                                               supported_versions)
